=== modified file 'NEWS'
--- NEWS	2009-04-03 21:50:40 +0000
+++ NEWS	2009-04-03 23:18:11 +0000
@@ -38,9 +38,12 @@
   respectively, but have support for filtered views disabled.
   (Ian Clatworthy)
 
-* New ``guess-renames`` command recognizes renames after they occur.
+* New ``mv --auto`` option recognizes renames after they occur.
   (Aaron Bentley)
 
+* ``bzr`` can now get passwords from stdin without requiring a controlling
+  terminal (i.e. by redirecting stdin). (Vincent Ladeuil)
+
 * ``bzr log`` now supports filtering of multiple files and directories
   and will show changes that touch any of them. Furthermore,
   directory filtering now shows the changes to any children of that
@@ -128,6 +131,9 @@
 * bzr gives a better message if an invalid regexp is passed to ``bzr log
   -m``.  (Anne Mohsen, Martin Pool)
 
+* ``bzr version-info`` now works in empty branches. (Jelmer Vernooij, 
+  #313028)
+
 * Fix "is not a stackable format" error when pushing a
   stackable-format branch with an unstackable-format repository to a
   destination with a default stacking policy.  (Andrew Bennetts)
@@ -242,6 +248,10 @@
 Internals
 *********
 
+* Added ``bzrlib.inventory_delta`` module.  This will be used for
+  serializing and deserializing inventory deltas for more efficient
+  streaming on the the network.  (Robert Collins, Andrew Bennetts)
+
 * ``Branch._get_config`` has been added, which splits out access to the
   specific config file from the branch. This is used to let RemoteBranch
   avoid constructing real branch objects to access configuration settings.
@@ -289,6 +299,11 @@
   handle existing svn properties that define a list of keywords to be
   expanded.  (Ian Clatworthy)
 
+* ``RemoteBranchConfig`` will use a new verb ``Branch.set_config_option``
+  to write config settings to smart servers that support this, saving
+  5 round trips on the stacked streaming acceptance test.
+  (Robert Collins, Andrew Bennetts)
+
 * ``RemoteBranch`` now provides ``_get_config`` for access to just the
   branch specific configuration from a remote server, which uses the 
   already existing ``Branch.get_config_file`` smart verb.

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2009-03-31 11:33:02 +0000
+++ bzrlib/builtins.py	2009-04-03 17:53:03 +0000
@@ -722,14 +722,19 @@
     takes_args = ['names*']
     takes_options = [Option("after", help="Move only the bzr identifier"
         " of the file, because the file has already been moved."),
+        Option('auto', help='Automatically guess renames.'),
+        Option('dry-run', help='Avoid making changes when guessing renames.'),
         ]
     aliases = ['move', 'rename']
     encoding_type = 'replace'
 
-    def run(self, names_list, after=False):
+    def run(self, names_list, after=False, auto=False, dry_run=False):
+        if auto:
+            return self.run_auto(names_list, after, dry_run)
+        elif dry_run:
+            raise errors.BzrCommandError('--dry-run requires --auto.')
         if names_list is None:
             names_list = []
-
         if len(names_list) < 2:
             raise errors.BzrCommandError("missing file argument")
         tree, rel_names = tree_files(names_list, canonicalize=False)
@@ -739,6 +744,20 @@
         finally:
             tree.unlock()
 
+    def run_auto(self, names_list, after, dry_run):
+        if names_list is not None and len(names_list) > 1:
+            raise errors.BzrCommandError('Only one path may be specified to'
+                                         ' --auto.')
+        if after:
+            raise errors.BzrCommandError('--after cannot be specified with'
+                                         ' --auto.')
+        work_tree, file_list = tree_files(names_list, default_branch='.')
+        work_tree.lock_write()
+        try:
+            rename_map.RenameMap.guess_renames(work_tree, dry_run)
+        finally:
+            work_tree.unlock()
+
     def _run(self, tree, names_list, rel_names, after):
         into_existing = osutils.isdir(names_list[-1])
         if into_existing and len(names_list) == 2:
@@ -2183,8 +2202,9 @@
             # evil when adding features", we continue to use the
             # original algorithm - per-file-graph - for the "single
             # file that isn't a directory without showing a delta" case.
+            partial_history = revision and b.repository._format.supports_chks
             match_using_deltas = (len(file_ids) != 1 or filter_by_dir
-                or delta_type)
+                or delta_type or partial_history)
 
             # Build the LogRequest and execute it
             if len(file_ids) == 0:
@@ -5268,22 +5288,6 @@
             urlutils.unescape_for_display(to_branch.base, 'utf-8'))
 
 
-class cmd_guess_renames(Command):
-    """Guess which files have been have been renamed, based on their content.
-
-    Only versioned files which have been deleted are candidates for rename
-    detection, and renames to ignored files will not be detected.
-    """
-
-    def run(self):
-        work_tree, file_list = tree_files(None, default_branch='.')
-        work_tree.lock_write()
-        try:
-            rename_map.RenameMap.guess_renames(work_tree)
-        finally:
-            work_tree.unlock()
-
-
 class cmd_view(Command):
     """Manage filtered views.
 

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2009-03-31 14:12:27 +0000
+++ bzrlib/bzrdir.py	2009-04-03 15:18:15 +0000
@@ -57,6 +57,9 @@
 from bzrlib.osutils import (
     sha_string,
     )
+from bzrlib.push import (
+    PushResult,
+    )
 from bzrlib.smart.client import _SmartClient
 from bzrlib.store.versioned import WeaveStore
 from bzrlib.transactions import WriteTransaction
@@ -1196,6 +1199,66 @@
                     basis.unlock()
         return result
 
+    def push_branch(self, source, revision_id=None, overwrite=False, 
+        remember=False):
+        """Push the source branch into this BzrDir."""
+        br_to = None
+        # If we can open a branch, use its direct repository, otherwise see
+        # if there is a repository without a branch.
+        try:
+            br_to = self.open_branch()
+        except errors.NotBranchError:
+            # Didn't find a branch, can we find a repository?
+            repository_to = self.find_repository()
+        else:
+            # Found a branch, so we must have found a repository
+            repository_to = br_to.repository
+
+        push_result = PushResult()
+        push_result.source_branch = source
+        if br_to is None:
+            # We have a repository but no branch, copy the revisions, and then
+            # create a branch.
+            repository_to.fetch(source.repository, revision_id=revision_id)
+            br_to = source.clone(self, revision_id=revision_id)
+            if source.get_push_location() is None or remember:
+                source.set_push_location(br_to.base)
+            push_result.stacked_on = None
+            push_result.branch_push_result = None
+            push_result.old_revno = None
+            push_result.old_revid = _mod_revision.NULL_REVISION
+            push_result.target_branch = br_to
+            push_result.master_branch = None
+            push_result.workingtree_updated = False
+        else:
+            # We have successfully opened the branch, remember if necessary:
+            if source.get_push_location() is None or remember:
+                source.set_push_location(br_to.base)
+            try:
+                tree_to = self.open_workingtree()
+            except errors.NotLocalUrl:
+                push_result.branch_push_result = source.push(br_to, 
+                    overwrite, stop_revision=revision_id)
+                push_result.workingtree_updated = False
+            except errors.NoWorkingTree:
+                push_result.branch_push_result = source.push(br_to,
+                    overwrite, stop_revision=revision_id)
+                push_result.workingtree_updated = None # Not applicable
+            else:
+                tree_to.lock_write()
+                try:
+                    push_result.branch_push_result = source.push(
+                        tree_to.branch, overwrite, stop_revision=revision_id)
+                    tree_to.update()
+                finally:
+                    tree_to.unlock()
+                push_result.workingtree_updated = True
+            push_result.old_revno = push_result.branch_push_result.old_revno
+            push_result.old_revid = push_result.branch_push_result.old_revid
+            push_result.target_branch = \
+                push_result.branch_push_result.target_branch
+        return push_result
+
 
 class BzrDirHooks(hooks.Hooks):
     """Hooks for BzrDir operations."""

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2009-03-27 04:36:58 +0000
+++ bzrlib/commit.py	2009-03-31 20:34:33 +0000
@@ -62,6 +62,7 @@
     revision,
     trace,
     tree,
+    xml_serializer,
     )
 from bzrlib.branch import Branch
 import bzrlib.config
@@ -607,14 +608,7 @@
         # serialiser not by commit. Then we can also add an unescaper
         # in the deserializer and start roundtripping revision messages
         # precisely. See repository_implementations/test_repository.py
-
-        # Python strings can include characters that can't be
-        # represented in well-formed XML; escape characters that
-        # aren't listed in the XML specification
-        # (http://www.w3.org/TR/REC-xml/#NT-Char).
-        self.message, escape_count = re.subn(
-            u'[^\x09\x0A\x0D\u0020-\uD7FF\uE000-\uFFFD]+',
-            lambda match: match.group(0).encode('unicode_escape'),
+        self.message, escape_count = xml_serializer.escape_invalid_chars(
             self.message)
         if escape_count:
             self.reporter.escaped(escape_count, self.message)

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2009-03-24 01:53:42 +0000
+++ bzrlib/errors.py	2009-04-02 10:43:01 +0000
@@ -2850,11 +2850,6 @@
     pass
 
 
-class NotATerminal(BzrError):
-
-    _fmt = 'Unable to ask for a password without real terminal.'
-
-
 class UnableEncodePath(BzrError):
 
     _fmt = ('Unable to encode %(kind)s path %(path)r in '

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2009-03-23 14:59:43 +0000
+++ bzrlib/inventory.py	2009-04-03 05:17:24 +0000
@@ -27,13 +27,13 @@
 # created, but it's not for now.
 ROOT_ID = "TREE_ROOT"
 
-import os
-import re
-import sys
+from copy import deepcopy
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 import collections
+import os
+import re
 import tarfile
 
 import bzrlib
@@ -877,6 +877,14 @@
             other.add(entry.copy())
         return other
 
+    def _get_mutable_inventory(self):
+        """Returns a mutable copy of the object.
+
+        Some inventories are immutable, yet working trees, for example, needs
+        to mutate exisiting inventories instead of creating a new one.
+        """
+        return deepcopy(self)
+
     def __iter__(self):
         return iter(self._byid)
 

=== added file 'bzrlib/inventory_delta.py'
--- bzrlib/inventory_delta.py	1970-01-01 00:00:00 +0000
+++ bzrlib/inventory_delta.py	2009-04-02 05:53:12 +0000
@@ -0,0 +1,299 @@
+# Copyright (C) 2008, 2009 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Inventory delta serialisation.
+
+See doc/developers/inventory.txt for the description of the format.
+
+In this module the interesting classes are:
+ - InventoryDeltaSerializer - object to read/write inventory deltas.
+"""
+
+__all__ = ['InventoryDeltaSerializer']
+
+from bzrlib import errors
+from bzrlib.osutils import basename
+from bzrlib import inventory
+from bzrlib.revision import NULL_REVISION
+
+
+def _directory_content(entry):
+    """Serialize the content component of entry which is a directory.
+    
+    :param entry: An InventoryDirectory.
+    """
+    return "dir"
+
+
+def _file_content(entry):
+    """Serialize the content component of entry which is a file.
+    
+    :param entry: An InventoryFile.
+    """
+    if entry.executable:
+        exec_bytes = 'Y'
+    else:
+        exec_bytes = ''
+    size_exec_sha = (entry.text_size, exec_bytes, entry.text_sha1)
+    if None in size_exec_sha:
+        raise errors.BzrError('Missing size or sha for %s' % entry.file_id)
+    return "file\x00%d\x00%s\x00%s" % size_exec_sha
+
+
+def _link_content(entry):
+    """Serialize the content component of entry which is a symlink.
+    
+    :param entry: An InventoryLink.
+    """
+    target = entry.symlink_target
+    if target is None:
+        raise errors.BzrError('Missing target for %s' % entry.file_id)
+    return "link\x00%s" % target.encode('utf8')
+
+
+def _reference_content(entry):
+    """Serialize the content component of entry which is a tree-reference.
+    
+    :param entry: A TreeReference.
+    """
+    tree_revision = entry.reference_revision
+    if tree_revision is None:
+        raise errors.BzrError('Missing reference revision for %s' % entry.file_id)
+    return "tree\x00%s" % tree_revision
+
+
+def _dir_to_entry(content, name, parent_id, file_id, last_modified,
+    _type=inventory.InventoryDirectory):
+    """Convert a dir content record to an InventoryDirectory."""
+    result = _type(file_id, name, parent_id)
+    result.revision = last_modified
+    return result
+
+
+def _file_to_entry(content, name, parent_id, file_id, last_modified,
+    _type=inventory.InventoryFile):
+    """Convert a dir content record to an InventoryFile."""
+    result = _type(file_id, name, parent_id)
+    result.revision = last_modified
+    result.text_size = int(content[1])
+    result.text_sha1 = content[3]
+    if content[2]:
+        result.executable = True
+    else:
+        result.executable = False
+    return result
+
+
+def _link_to_entry(content, name, parent_id, file_id, last_modified,
+    _type=inventory.InventoryLink):
+    """Convert a link content record to an InventoryLink."""
+    result = _type(file_id, name, parent_id)
+    result.revision = last_modified
+    result.symlink_target = content[1].decode('utf8')
+    return result
+
+
+def _tree_to_entry(content, name, parent_id, file_id, last_modified,
+    _type=inventory.TreeReference):
+    """Convert a tree content record to a TreeReference."""
+    result = _type(file_id, name, parent_id)
+    result.revision = last_modified
+    result.reference_revision = content[1]
+    return result
+
+
+
+class InventoryDeltaSerializer(object):
+    """Serialize and deserialize inventory deltas."""
+
+    FORMAT_1 = 'bzr inventory delta v1 (bzr 1.14)'
+
+    def __init__(self, versioned_root, tree_references):
+        """Create an InventoryDeltaSerializer.
+
+        :param versioned_root: If True, any root entry that is seen is expected
+            to be versioned, and root entries can have any fileid.
+        :param tree_references: If True support tree-reference entries.
+        """
+        self._versioned_root = versioned_root
+        self._tree_references = tree_references
+        self._entry_to_content = {
+            'directory': _directory_content,
+            'file': _file_content,
+            'symlink': _link_content,
+        }
+        if tree_references:
+            self._entry_to_content['tree-reference'] = _reference_content
+
+    def delta_to_lines(self, old_name, new_name, delta_to_new):
+        """Return a line sequence for delta_to_new.
+
+        :param old_name: A UTF8 revision id for the old inventory.  May be
+            NULL_REVISION if there is no older inventory and delta_to_new
+            includes the entire inventory contents.
+        :param new_name: The version name of the inventory we create with this
+            delta.
+        :param delta_to_new: An inventory delta such as Inventory.apply_delta
+            takes.
+        :return: The serialized delta as lines.
+        """
+        lines = ['', '', '', '', '']
+        to_line = self._delta_item_to_line
+        for delta_item in delta_to_new:
+            lines.append(to_line(delta_item))
+            if lines[-1].__class__ != str:
+                raise errors.BzrError(
+                    'to_line generated non-str output %r' % lines[-1])
+        lines.sort()
+        lines[0] = "format: %s\n" % InventoryDeltaSerializer.FORMAT_1
+        lines[1] = "parent: %s\n" % old_name
+        lines[2] = "version: %s\n" % new_name
+        lines[3] = "versioned_root: %s\n" % self._serialize_bool(
+            self._versioned_root)
+        lines[4] = "tree_references: %s\n" % self._serialize_bool(
+            self._tree_references)
+        return lines
+
+    def _serialize_bool(self, value):
+        if value:
+            return "true"
+        else:
+            return "false"
+
+    def _delta_item_to_line(self, delta_item):
+        """Convert delta_item to a line."""
+        oldpath, newpath, file_id, entry = delta_item
+        if newpath is None:
+            # delete
+            oldpath_utf8 = '/' + oldpath.encode('utf8')
+            newpath_utf8 = 'None'
+            parent_id = ''
+            last_modified = NULL_REVISION
+            content = 'deleted\x00\x00'
+        else:
+            if oldpath is None:
+                oldpath_utf8 = 'None'
+            else:
+                oldpath_utf8 = '/' + oldpath.encode('utf8')
+            # TODO: Test real-world utf8 cache hit rate. It may be a win.
+            newpath_utf8 = '/' + newpath.encode('utf8')
+            # Serialize None as ''
+            parent_id = entry.parent_id or ''
+            # Serialize unknown revisions as NULL_REVISION
+            last_modified = entry.revision
+            # special cases for /
+            if newpath_utf8 == '/' and not self._versioned_root:
+                if file_id != 'TREE_ROOT':
+                    raise errors.BzrError(
+                        'file_id %s is not TREE_ROOT for /' % file_id)
+                if last_modified is not None:
+                    raise errors.BzrError(
+                        'Version present for / in %s' % file_id)
+                last_modified = NULL_REVISION
+            if last_modified is None:
+                raise errors.BzrError("no version for fileid %s" % file_id)
+            content = self._entry_to_content[entry.kind](entry)
+        return ("%s\x00%s\x00%s\x00%s\x00%s\x00%s\n" %
+            (oldpath_utf8, newpath_utf8, file_id, parent_id, last_modified,
+                content))
+
+    def _deserialize_bool(self, value):
+        if value == "true":
+            return True
+        elif value == "false":
+            return False
+        else:
+            raise errors.BzrError("value %r is not a bool" % (value,))
+
+    def parse_text_bytes(self, bytes):
+        """Parse the text bytes of a serialized inventory delta.
+
+        :param bytes: The bytes to parse. This can be obtained by calling
+            delta_to_lines and then doing ''.join(delta_lines).
+        :return: (parent_id, new_id, inventory_delta)
+        """
+        lines = bytes.split('\n')[:-1] # discard the last empty line
+        if not lines or lines[0] != 'format: %s' % InventoryDeltaSerializer.FORMAT_1:
+            raise errors.BzrError('unknown format %r' % lines[0:1])
+        if len(lines) < 2 or not lines[1].startswith('parent: '):
+            raise errors.BzrError('missing parent: marker')
+        delta_parent_id = lines[1][8:]
+        if len(lines) < 3 or not lines[2].startswith('version: '):
+            raise errors.BzrError('missing version: marker')
+        delta_version_id = lines[2][9:]
+        if len(lines) < 4 or not lines[3].startswith('versioned_root: '):
+            raise errors.BzrError('missing versioned_root: marker')
+        delta_versioned_root = self._deserialize_bool(lines[3][16:])
+        if len(lines) < 5 or not lines[4].startswith('tree_references: '):
+            raise errors.BzrError('missing tree_references: marker')
+        delta_tree_references = self._deserialize_bool(lines[4][17:])
+        if delta_versioned_root != self._versioned_root:
+            raise errors.BzrError(
+                "serialized versioned_root flag is wrong: %s" %
+                (delta_versioned_root,))
+        if delta_tree_references != self._tree_references:
+            raise errors.BzrError(
+                "serialized tree_references flag is wrong: %s" %
+                (delta_tree_references,))
+        result = []
+        seen_ids = set()
+        line_iter = iter(lines)
+        for i in range(5):
+            line_iter.next()
+        for line in line_iter:
+            (oldpath_utf8, newpath_utf8, file_id, parent_id, last_modified,
+                content) = line.split('\x00', 5)
+            parent_id = parent_id or None
+            if file_id in seen_ids:
+                raise errors.BzrError(
+                    "duplicate file id in inventory delta %r" % lines)
+            seen_ids.add(file_id)
+            if newpath_utf8 == '/' and not delta_versioned_root and (
+                last_modified != 'null:' or file_id != 'TREE_ROOT'):
+                    raise errors.BzrError("Versioned root found: %r" % line)
+            elif last_modified[-1] == ':':
+                    raise errors.BzrError('special revisionid found: %r' % line)
+            if not delta_tree_references and content.startswith('tree\x00'):
+                raise errors.BzrError("Tree reference found: %r" % line)
+            content_tuple = tuple(content.split('\x00'))
+            entry = _parse_entry(
+                newpath_utf8, file_id, parent_id, last_modified, content_tuple)
+            if oldpath_utf8 == 'None':
+                oldpath = None
+            else:
+                oldpath = oldpath_utf8.decode('utf8')
+            if newpath_utf8 == 'None':
+                newpath = None
+            else:
+                newpath = newpath_utf8.decode('utf8')
+            delta_item = (oldpath, newpath, file_id, entry)
+            result.append(delta_item)
+        return delta_parent_id, delta_version_id, result
+
+
+def _parse_entry(utf8_path, file_id, parent_id, last_modified, content):
+    entry_factory = {
+        'dir': _dir_to_entry,
+        'file': _file_to_entry,
+        'link': _link_to_entry,
+        'tree': _tree_to_entry,
+    }
+    kind = content[0]
+    path = utf8_path[1:].decode('utf8')
+    name = basename(path)
+    return entry_factory[content[0]](
+            content, name, parent_id, file_id, last_modified)
+

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2009-03-25 02:03:41 +0000
+++ bzrlib/knit.py	2009-04-03 00:53:29 +0000
@@ -955,9 +955,13 @@
                 lines[-1] = lines[-1] + '\n'
                 line_bytes += '\n'
 
-        for element in key:
+        for element in key[:-1]:
             if type(element) != str:
                 raise TypeError("key contains non-strings: %r" % (key,))
+        if key[-1] is None:
+            key = key[:-1] + ('sha1:' + digest,)
+        elif type(key[-1]) != str:
+                raise TypeError("key contains non-strings: %r" % (key,))
         # Knit hunks are still last-element only
         version_id = key[-1]
         content = self._factory.make(lines, version_id)
@@ -1022,9 +1026,10 @@
     def _check_add(self, key, lines, random_id, check_content):
         """check that version_id and lines are safe to add."""
         version_id = key[-1]
-        if contains_whitespace(version_id):
-            raise InvalidRevisionId(version_id, self)
-        self.check_not_reserved_id(version_id)
+        if version_id is not None:
+            if contains_whitespace(version_id):
+                raise InvalidRevisionId(version_id, self)
+            self.check_not_reserved_id(version_id)
         # TODO: If random_id==False and the key is already present, we should
         # probably check that the existing content is identical to what is
         # being inserted, and otherwise raise an exception.  This would make

=== modified file 'bzrlib/memorytree.py'
--- bzrlib/memorytree.py	2009-03-24 23:19:12 +0000
+++ bzrlib/memorytree.py	2009-04-03 05:17:24 +0000
@@ -20,7 +20,6 @@
 """
 
 
-from copy import deepcopy
 import os
 
 from bzrlib import (
@@ -217,7 +216,7 @@
             self._parent_ids = []
         else:
             self._parent_ids = [self._branch_revision_id]
-        self._inventory = deepcopy(self._basis_tree._inventory)
+        self._inventory = self._basis_tree._inventory._get_mutable_inventory()
         self._file_transport = MemoryTransport()
         # TODO copy the revision trees content, or do it lazy, or something.
         inventory_entries = self._inventory.iter_entries()

=== modified file 'bzrlib/mutabletree.py'
--- bzrlib/mutabletree.py	2009-03-24 12:15:01 +0000
+++ bzrlib/mutabletree.py	2009-04-03 05:17:24 +0000
@@ -552,7 +552,10 @@
         # WorkingTree classes for optimised versions for specific format trees.
         basis = self.basis_tree()
         basis.lock_read()
-        inventory = basis.inventory
+        # TODO: Consider re-evaluating the need for this with CHKInventory
+        # we don't strictly need to mutate an inventory for this
+        # it only makes sense when apply_delta is cheaper than get_inventory()
+        inventory = basis.inventory._get_mutable_inventory()
         basis.unlock()
         inventory.apply_delta(delta)
         rev_tree = RevisionTree(self.branch.repository, inventory, new_revid)

=== modified file 'bzrlib/push.py'
--- bzrlib/push.py	2009-03-23 14:59:43 +0000
+++ bzrlib/push.py	2009-04-03 00:02:29 +0000
@@ -16,21 +16,34 @@
 
 """UI helper for the push command."""
 
-from bzrlib import (builtins, bzrdir, errors, revision as _mod_revision,
-                    transport)
-from bzrlib.trace import note, warning
+from bzrlib import (
+    builtins,
+    branch,
+    bzrdir,
+    errors,
+    revision as _mod_revision,
+    transport,
+    )
+from bzrlib.trace import (
+    note,
+    warning,
+    )
 
 
 class PushResult(object):
     """Result of a push operation.
 
     :ivar branch_push_result: Result of a push between branches
+    :ivar target_branch: The target branch
     :ivar stacked_on: URL of the branch on which the result is stacked
+    :ivar workingtree_updated: Whether or not the target workingtree was updated.
     """
 
     def __init__(self):
         self.branch_push_result = None
         self.stacked_on = None
+        self.workingtree_updated = None
+        self.target_branch = None
 
     def report(self, to_file):
         """Write a human-readable description of the result."""
@@ -71,20 +84,6 @@
         dir_to = bzrdir.BzrDir.open_from_transport(to_transport)
     except errors.NotBranchError:
         pass # Didn't find anything
-    else:
-        # If we can open a branch, use its direct repository, otherwise see
-        # if there is a repository without a branch.
-        try:
-            br_to = dir_to.open_branch()
-        except errors.NotBranchError:
-            # Didn't find a branch, can we find a repository?
-            try:
-                repository_to = dir_to.find_repository()
-            except errors.NoRepositoryPresent:
-                pass
-        else:
-            # Found a branch, so we must have found a repository
-            repository_to = br_to.repository
 
     push_result = PushResult()
     if dir_to is None:
@@ -133,69 +132,41 @@
                 errors.UnstackableRepositoryFormat,
                 errors.NotStacked):
             push_result.stacked_on = None
-        # We successfully created the target, remember it
+        push_result.target_branch = br_to
+        push_result.old_revid = _mod_revision.NULL_REVISION
+        push_result.old_revno = 0
         if br_from.get_push_location() is None or remember:
             br_from.set_push_location(br_to.base)
-    elif repository_to is None:
-        # we have a bzrdir but no branch or repository
-        # XXX: Figure out what to do other than complain.
-        raise errors.BzrCommandError("At %s you have a valid .bzr control"
-            " directory, but not a branch or repository. This is an"
-            " unsupported configuration. Please move the target directory"
-            " out of the way and try again."
-            % location)
-    elif br_to is None:
-        # We have a repository but no branch, copy the revisions, and then
-        # create a branch.
+    else:
         if stacked_on is not None:
             warning("Ignoring request for a stacked branch as repository "
                     "already exists at the destination location.")
-        repository_to.fetch(br_from.repository, revision_id=revision_id)
-        br_to = br_from.clone(dir_to, revision_id=revision_id)
-        if br_from.get_push_location() is None or remember:
-            br_from.set_push_location(br_to.base)
-    else: # We have a valid to branch
-        if stacked_on is not None:
-            warning("Ignoring request for a stacked branch as branch "
-                    "already exists at the destination location.")
-        # We were able to connect to the remote location, so remember it.
-        # (We don't need to successfully push because of possible divergence.)
-        if br_from.get_push_location() is None or remember:
-            br_from.set_push_location(br_to.base)
         try:
-            try:
-                tree_to = dir_to.open_workingtree()
-            except errors.NotLocalUrl:
-                note("This transport does not update the working "
-                        "tree of: %s. See 'bzr help working-trees' for "
-                        "more information." % br_to.base)
-                push_result.branch_push_result = br_from.push(br_to, overwrite,
-                                                   stop_revision=revision_id)
-            except errors.NoWorkingTree:
-                push_result.branch_push_result = br_from.push(br_to, overwrite,
-                                                   stop_revision=revision_id)
-            else:
-                tree_to.lock_write()
-                try:
-                    push_result.branch_push_result = br_from.push(tree_to.branch,
-                        overwrite, stop_revision=revision_id)
-                    tree_to.update()
-                finally:
-                    tree_to.unlock()
+            push_result = dir_to.push_branch(br_from, revision_id, overwrite, 
+                remember)
         except errors.DivergedBranches:
             raise errors.BzrCommandError('These branches have diverged.'
                                     '  Try using "merge" and then "push".')
+        except errors.NoRepositoryPresent:
+            # we have a bzrdir but no branch or repository
+            # XXX: Figure out what to do other than complain.
+            raise errors.BzrCommandError("At %s you have a valid .bzr"
+                " control directory, but not a branch or repository. This"
+                " is an unsupported configuration. Please move the target"
+                " directory out of the way and try again." % location)
+        if push_result.workingtree_updated == False:
+            warning("This transport does not update the working " 
+                    "tree of: %s. See 'bzr help working-trees' for "
+                    "more information." % push_result.target_branch.base)
     push_result.report(to_file)
-    if push_result.branch_push_result is not None:
-        old_revid = push_result.branch_push_result.old_revid
-        old_revno = push_result.branch_push_result.old_revno
-    else:
-        old_revid = _mod_revision.NULL_REVISION
-        old_revno = 0
     if verbose:
+        br_to = push_result.target_branch
         br_to.lock_read()
         try:
             from bzrlib.log import show_branch_change
-            show_branch_change(br_to, to_file, old_revno, old_revid)
+            show_branch_change(br_to, to_file, push_result.old_revno, 
+                               push_result.old_revid)
         finally:
             br_to.unlock()
+
+

=== modified file 'bzrlib/reconcile.py'
--- bzrlib/reconcile.py	2009-03-23 14:59:43 +0000
+++ bzrlib/reconcile.py	2009-04-03 07:18:11 +0000
@@ -505,9 +505,8 @@
             total_inventories = len(list(
                 collection.inventory_index.combined_index.iter_all_entries()))
             if len(all_revisions):
-                self._packer = repofmt.pack_repo.ReconcilePacker(
-                    collection, packs, ".reconcile", all_revisions)
-                new_pack = self._packer.pack(pb=self.pb)
+                new_pack =  self.repo._reconcile_pack(collection, packs,
+                    ".reconcile", all_revisions, self.pb)
                 if new_pack is not None:
                     self._discard_and_save(packs)
             else:

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2009-04-02 03:37:30 +0000
+++ bzrlib/remote.py	2009-04-03 02:08:31 +0000
@@ -917,12 +917,26 @@
         if isinstance(repository, RemoteRepository):
             raise AssertionError()
         self._real_repository = repository
-        # If the _real_repository has _fallback_repositories, clear them out,
-        # because we want it to have the same set as this repository.  This is
-        # reasonable to do because the fallbacks we clear here are from a
-        # "real" branch, and we're about to replace them with the equivalents
-        # from a RemoteBranch.
-        self._real_repository._fallback_repositories = []
+        # three code paths happen here:
+        # 1) old servers, RemoteBranch.open() calls _ensure_real before setting
+        # up stacking. In this case self._fallback_repositories is [], and the
+        # real repo is already setup. Preserve the real repo and
+        # RemoteRepository.add_fallback_repository will avoid adding
+        # duplicates.
+        # 2) new servers, RemoteBranch.open() sets up stacking, and when
+        # ensure_real is triggered from a branch, the real repository to
+        # set already has a matching list with separate instances, but
+        # as they are also RemoteRepositories we don't worry about making the
+        # lists be identical.
+        # 3) new servers, RemoteRepository.ensure_real is triggered before
+        # RemoteBranch.ensure real, in this case we get a repo with no fallbacks
+        # and need to populate it.
+        if (self._fallback_repositories and
+            len(self._real_repository._fallback_repositories) !=
+            len(self._fallback_repositories)):
+            if len(self._real_repository._fallback_repositories):
+                raise AssertionError(
+                    "cannot cleanly remove existing _fallback_repositories")
         for fb in self._fallback_repositories:
             self._real_repository.add_fallback_repository(fb)
         if self._lock_mode == 'w':
@@ -1057,7 +1071,9 @@
         # _real_branch had its get_stacked_on_url method called), then the
         # repository to be added may already be in the _real_repositories list.
         if self._real_repository is not None:
-            if repository not in self._real_repository._fallback_repositories:
+            fallback_locations = [repo.bzrdir.root_transport.base for repo in
+                self._real_repository._fallback_repositories]
+            if repository.bzrdir.root_transport.base not in fallback_locations:
                 self._real_repository.add_fallback_repository(repository)
 
     def add_inventory(self, revid, inv, parents):
@@ -2389,14 +2405,13 @@
         return section_obj.get(name, default)
 
     def _get_configobj(self):
-        path = self._branch.bzrdir._path_for_remote_call(
-            self._branch._client)
+        path = self._branch._remote_path()
         response = self._branch._client.call_expecting_body(
             'Branch.get_config_file', path)
         if response[0][0] != 'ok':
             raise UnexpectedSmartServerResponse(response)
-        bytes = response[1].read_body_bytes()
-        return config.ConfigObj([bytes], encoding='utf-8')
+        lines = response[1].read_body_bytes().splitlines()
+        return config.ConfigObj(lines, encoding='utf-8')
 
     def set_option(self, value, name, section=None):
         """Set the value associated with a named option.
@@ -2405,7 +2420,19 @@
         :param name: The name of the value to set
         :param section: The section the option is in (if any)
         """
-        return self._vfs_set_option(value, name, section)
+        medium = self._branch._client._medium
+        if medium._is_remote_before((1, 14)):
+            return self._vfs_set_option(value, name, section)
+        try:
+            path = self._branch._remote_path()
+            response = self._branch._client.call('Branch.set_config_option',
+                path, self._branch._lock_token, self._branch._repo_lock_token,
+                value.encode('utf8'), name, section or '')
+        except errors.UnknownSmartMethod:
+            medium._remember_remote_is_before((1, 14))
+            return self._vfs_set_option(value, name, section)
+        if response != ():
+            raise errors.UnexpectedSmartServerResponse(response)
 
     def _vfs_set_option(self, value, name, section=None):
         self._branch._ensure_real()

=== modified file 'bzrlib/rename_map.py'
--- bzrlib/rename_map.py	2009-03-24 12:54:05 +0000
+++ bzrlib/rename_map.py	2009-03-24 15:47:32 +0000
@@ -20,6 +20,7 @@
 from bzrlib import (
     osutils,
     progress,
+    trace,
 )
 from bzrlib.ui import ui_factory
 
@@ -202,7 +203,7 @@
         return missing_files, missing_parents, candidate_files
 
     @classmethod
-    def guess_renames(klass, tree):
+    def guess_renames(klass, tree, dry_run=False):
         """Guess which files to rename, and perform the rename.
 
         We assume that unversioned files and missing files indicate that
@@ -235,12 +236,16 @@
                                                    missing_parents)
                 matches.update(parents_matches)
             pp.next_phase()
-            rn._update_tree(required_parents, matches)
+            delta = rn._make_inventory_delta(matches)
+            for old, new, file_id, entry in delta:
+                trace.note("%s => %s", old, new)
+            if not dry_run:
+                tree.add(required_parents)
+                tree.apply_inventory_delta(delta)
         finally:
             task.finished()
 
-    def _update_tree(self, required_parents, matches):
-        self.tree.add(required_parents)
+    def _make_inventory_delta(self, matches):
         delta = []
         file_id_matches = dict((f, p) for p, f in matches.items())
         for old_path, entry in self.tree.iter_entries_by_dir(matches.values()):
@@ -249,8 +254,10 @@
             parent_id = matches.get(parent_path)
             if parent_id is None:
                 parent_id = self.tree.path2id(parent_path)
+            if entry.name == new_name and entry.parent_id == parent_id:
+                continue
             new_entry = entry.copy()
             new_entry.parent_id = parent_id
             new_entry.name = new_name
             delta.append((old_path, new_path, new_entry.file_id, new_entry))
-        self.tree.apply_inventory_delta(delta)
+        return delta

=== modified file 'bzrlib/repofmt/knitrepo.py'
--- bzrlib/repofmt/knitrepo.py	2009-03-24 01:53:42 +0000
+++ bzrlib/repofmt/knitrepo.py	2009-04-03 13:08:27 +0000
@@ -291,6 +291,8 @@
     supports_ghosts = True
     # External lookups are not supported in this format.
     supports_external_lookups = False
+    # No CHK support.
+    supports_chks = False
     _fetch_order = 'topological'
     _fetch_uses_deltas = True
     fast_deltas = False
@@ -379,6 +381,7 @@
         repo.signatures = self._get_signatures(repo_transport, repo)
         repo.inventories = self._get_inventories(repo_transport, repo)
         repo.texts = self._get_texts(repo_transport, repo)
+        repo.chk_bytes = None
         repo._transport = repo_transport
         return repo
 

=== modified file 'bzrlib/repofmt/pack_repo.py'
--- bzrlib/repofmt/pack_repo.py	2009-03-25 04:20:12 +0000
+++ bzrlib/repofmt/pack_repo.py	2009-04-03 13:08:27 +0000
@@ -2071,6 +2071,7 @@
                 deltas=True, parents=True, is_locked=self.is_locked),
             data_access=self._pack_collection.text_index.data_access,
             max_delta_chain=200)
+        self.chk_bytes = None
         # True when the repository object is 'write locked' (as opposed to the
         # physical lock only taken out around changes to the pack-names list.)
         # Another way to represent this would be a decorator around the control
@@ -2223,6 +2224,10 @@
         reconciler.reconcile()
         return reconciler
 
+    def _reconcile_pack(self, collection, packs, extension, revs, pb):
+        packer = ReconcilePacker(collection, packs, extension, revs)
+        return packer.pack(pb)
+
     def unlock(self):
         if self._write_lock_count == 1 and self._write_group is not None:
             self.abort_write_group()
@@ -2274,6 +2279,8 @@
     supports_ghosts = True
     # External references are not supported in pack repositories yet.
     supports_external_lookups = False
+    # Most pack formats do not use chk lookups.
+    supports_chks = False
     # What index classes to use
     index_builder_class = None
     index_class = None

=== modified file 'bzrlib/repofmt/weaverepo.py'
--- bzrlib/repofmt/weaverepo.py	2009-03-25 04:20:12 +0000
+++ bzrlib/repofmt/weaverepo.py	2009-04-03 13:08:27 +0000
@@ -266,6 +266,7 @@
     supports_tree_reference = False
     supports_ghosts = False
     supports_external_lookups = False
+    supports_chks = False
     _fetch_order = 'topological'
     _fetch_reconcile = True
     fast_deltas = False
@@ -315,6 +316,7 @@
         result.signatures = self._get_signatures(repo_transport, result)
         result.inventories = self._get_inventories(repo_transport, result)
         result.texts = self._get_texts(repo_transport, result)
+        result.chk_bytes = None
         return result
 
     def check_conversion_target(self, target_format):
@@ -475,6 +477,8 @@
 
     _versionedfile_class = weave.WeaveFile
     supports_ghosts = False
+    supports_chks = False
+
     _fetch_order = 'topological'
     _fetch_reconcile = True
     fast_deltas = False
@@ -555,6 +559,7 @@
         result.signatures = self._get_signatures(repo_transport, result)
         result.inventories = self._get_inventories(repo_transport, result)
         result.texts = self._get_texts(repo_transport, result)
+        result.chk_bytes = None
         result._transport = repo_transport
         return result
 

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2009-04-01 15:14:38 +0000
+++ bzrlib/repository.py	2009-04-02 10:21:08 +0000
@@ -16,6 +16,7 @@
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
+import cStringIO
 import re
 import time
 
@@ -1635,13 +1636,20 @@
 
     @needs_read_lock
     def get_revision_xml(self, revision_id):
-        """Return the XML representation of a revision.
-
-        :param revision_id: Revision for which to return the XML.
-        :return: XML string
-        """
-        return self._serializer.write_revision_to_string(
-            self.get_revision(revision_id))
+        # TODO: jam 20070210 This shouldn't be necessary since get_revision
+        #       would have already do it.
+        # TODO: jam 20070210 Just use _serializer.write_revision_to_string()
+        # TODO: this can't just be replaced by:
+        # return self._serializer.write_revision_to_string(
+        #     self.get_revision(revision_id))
+        # as cStringIO preservers the encoding unlike write_revision_to_string
+        # or some other call down the path.
+        rev = self.get_revision(revision_id)
+        rev_tmp = cStringIO.StringIO()
+        # the current serializer..
+        self._serializer.write_revision(rev, rev_tmp)
+        rev_tmp.seek(0)
+        return rev_tmp.getvalue()
 
     def get_deltas_for_revisions(self, revisions, specific_fileids=None):
         """Produce a generator of revision deltas.

=== modified file 'bzrlib/smart/branch.py'
--- bzrlib/smart/branch.py	2009-04-02 03:37:30 +0000
+++ bzrlib/smart/branch.py	2009-04-03 00:37:53 +0000
@@ -148,6 +148,16 @@
             return FailedSmartServerResponse(('TipChangeRejected', msg))
 
 
+class SmartServerBranchRequestSetConfigOption(SmartServerLockedBranchRequest):
+    """Set an option in the branch configuration."""
+
+    def do_with_locked_branch(self, branch, value, name, section):
+        if not section:
+            section = None
+        branch._get_config().set_option(value.decode('utf8'), name, section)
+        return SuccessfulSmartServerResponse(())
+
+
 class SmartServerBranchRequestSetLastRevision(SmartServerSetTipRequest):
 
     def do_tip_change_with_locked_branch(self, branch, new_last_revision_id):

=== modified file 'bzrlib/smart/request.py'
--- bzrlib/smart/request.py	2009-03-26 06:59:15 +0000
+++ bzrlib/smart/request.py	2009-04-02 05:22:42 +0000
@@ -448,10 +448,12 @@
     'Branch.last_revision_info', 'bzrlib.smart.branch', 'SmartServerBranchRequestLastRevisionInfo')
 request_handlers.register_lazy(
     'Branch.lock_write', 'bzrlib.smart.branch', 'SmartServerBranchRequestLockWrite')
-request_handlers.register_lazy(
-    'Branch.revision_history', 'bzrlib.smart.branch', 'SmartServerRequestRevisionHistory')
-request_handlers.register_lazy(
-    'Branch.set_last_revision', 'bzrlib.smart.branch', 'SmartServerBranchRequestSetLastRevision')
+request_handlers.register_lazy( 'Branch.revision_history',
+    'bzrlib.smart.branch', 'SmartServerRequestRevisionHistory')
+request_handlers.register_lazy( 'Branch.set_config_option',
+    'bzrlib.smart.branch', 'SmartServerBranchRequestSetConfigOption')
+request_handlers.register_lazy( 'Branch.set_last_revision',
+    'bzrlib.smart.branch', 'SmartServerBranchRequestSetLastRevision')
 request_handlers.register_lazy(
     'Branch.set_last_revision_info', 'bzrlib.smart.branch',
     'SmartServerBranchRequestSetLastRevisionInfo')

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2009-04-03 21:50:40 +0000
+++ bzrlib/tests/__init__.py	2009-04-03 23:18:11 +0000
@@ -675,25 +675,14 @@
     Allows get_password to be tested without real tty attached.
     """
 
-    def __init__(self,
-                 stdout=None,
-                 stderr=None,
-                 stdin=None):
-        super(TestUIFactory, self).__init__()
+    def __init__(self, stdout=None, stderr=None, stdin=None):
         if stdin is not None:
             # We use a StringIOWrapper to be able to test various
             # encodings, but the user is still responsible to
             # encode the string and to set the encoding attribute
             # of StringIOWrapper.
-            self.stdin = StringIOWrapper(stdin)
-        if stdout is None:
-            self.stdout = sys.stdout
-        else:
-            self.stdout = stdout
-        if stderr is None:
-            self.stderr = sys.stderr
-        else:
-            self.stderr = stderr
+            stdin = StringIOWrapper(stdin)
+        super(TestUIFactory, self).__init__(stdin, stdout, stderr)
 
     def clear(self):
         """See progress.ProgressBar.clear()."""
@@ -701,9 +690,6 @@
     def clear_term(self):
         """See progress.ProgressBar.clear_term()."""
 
-    def clear_term(self):
-        """See progress.ProgressBar.clear_term()."""
-
     def finished(self):
         """See progress.ProgressBar.finished()."""
 
@@ -720,10 +706,8 @@
     def update(self, message, count=None, total=None):
         """See progress.ProgressBar.update()."""
 
-    def get_non_echoed_password(self, prompt):
+    def get_non_echoed_password(self):
         """Get password from stdin without trying to handle the echo mode"""
-        if prompt:
-            self.stdout.write(prompt.encode(self.stdout.encoding, 'replace'))
         password = self.stdin.readline()
         if not password:
             raise EOFError
@@ -3345,6 +3329,7 @@
                    'bzrlib.tests.test_index',
                    'bzrlib.tests.test_info',
                    'bzrlib.tests.test_inv',
+                   'bzrlib.tests.test_inventory_delta',
                    'bzrlib.tests.test_knit',
                    'bzrlib.tests.test_lazy_import',
                    'bzrlib.tests.test_lazy_regex',

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2009-03-24 12:15:01 +0000
+++ bzrlib/tests/blackbox/__init__.py	2009-04-01 20:30:51 +0000
@@ -59,7 +59,6 @@
                      'bzrlib.tests.blackbox.test_filesystem_cicp',
                      'bzrlib.tests.blackbox.test_filtered_view_ops',
                      'bzrlib.tests.blackbox.test_find_merge_base',
-                     'bzrlib.tests.blackbox.test_guess_renames',
                      'bzrlib.tests.blackbox.test_help',
                      'bzrlib.tests.blackbox.test_hooks',
                      'bzrlib.tests.blackbox.test_ignore',

=== removed file 'bzrlib/tests/blackbox/test_guess_renames.py'
--- bzrlib/tests/blackbox/test_guess_renames.py	2009-03-24 12:54:05 +0000
+++ bzrlib/tests/blackbox/test_guess_renames.py	1970-01-01 00:00:00 +0000
@@ -1,24 +0,0 @@
-# Copyright (C) 2008 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-
-from bzrlib import tests
-
-
-class TestGuessRenames(tests.TestCaseWithTransport):
-
-    def test_guess_renames(self):
-        self.run_bzr('guess-renames')

=== modified file 'bzrlib/tests/blackbox/test_mv.py'
--- bzrlib/tests/blackbox/test_mv.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_mv.py	2009-04-01 20:30:51 +0000
@@ -430,3 +430,58 @@
         self.assertNotInWorkingTree('c')
         self.failUnlessExists('d')
         self.assertInWorkingTree('d')
+
+    def make_abcd_tree(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a', 'tree/c'])
+        tree.add(['a', 'c'])
+        tree.commit('record old names')
+        osutils.rename('tree/a', 'tree/b')
+        osutils.rename('tree/c', 'tree/d')
+        return tree
+
+    def test_mv_auto(self):
+        self.make_abcd_tree()
+        out, err = self.run_bzr('mv --auto', working_dir='tree')
+        self.assertEqual(out, '')
+        self.assertEqual(err, 'a => b\nc => d\n')
+        tree = workingtree.WorkingTree.open('tree')
+        self.assertIsNot(None, tree.path2id('b'))
+        self.assertIsNot(None, tree.path2id('d'))
+
+    def test_mv_auto_one_path(self):
+        self.make_abcd_tree()
+        out, err = self.run_bzr('mv --auto tree')
+        self.assertEqual(out, '')
+        self.assertEqual(err, 'a => b\nc => d\n')
+        tree = workingtree.WorkingTree.open('tree')
+        self.assertIsNot(None, tree.path2id('b'))
+        self.assertIsNot(None, tree.path2id('d'))
+
+    def test_mv_auto_two_paths(self):
+        self.make_abcd_tree()
+        out, err = self.run_bzr('mv --auto tree tree2', retcode=3)
+        self.assertEqual('bzr: ERROR: Only one path may be specified to'
+                         ' --auto.\n', err)
+
+    def test_mv_auto_dry_run(self):
+        self.make_abcd_tree()
+        out, err = self.run_bzr('mv --auto --dry-run', working_dir='tree')
+        self.assertEqual(out, '')
+        self.assertEqual(err, 'a => b\nc => d\n')
+        tree = workingtree.WorkingTree.open('tree')
+        self.assertIsNot(None, tree.path2id('a'))
+        self.assertIsNot(None, tree.path2id('c'))
+
+    def test_mv_no_auto_dry_run(self):
+        self.make_abcd_tree()
+        out, err = self.run_bzr('mv c d --dry-run',
+                                working_dir='tree', retcode=3)
+        self.assertEqual('bzr: ERROR: --dry-run requires --auto.\n', err)
+
+    def test_mv_auto_after(self):
+        self.make_abcd_tree()
+        out, err = self.run_bzr('mv --auto --after', working_dir='tree',
+                                retcode=3)
+        self.assertEqual('bzr: ERROR: --after cannot be specified with'
+                         ' --auto.\n', err)

=== modified file 'bzrlib/tests/blackbox/test_push.py'
--- bzrlib/tests/blackbox/test_push.py	2009-04-02 04:01:23 +0000
+++ bzrlib/tests/blackbox/test_push.py	2009-04-03 19:03:38 +0000
@@ -217,7 +217,7 @@
         # being too low. If rpc_count increases, more network roundtrips have
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(47, self.hpss_calls)
+        self.assertLength(42, self.hpss_calls)
         remote = Branch.open('public')
         self.assertEndsWith(remote.get_stacked_on_url(), '/parent')
 

=== modified file 'bzrlib/tests/bzrdir_implementations/__init__.py'
--- bzrlib/tests/bzrdir_implementations/__init__.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/bzrdir_implementations/__init__.py	2009-04-03 14:24:40 +0000
@@ -76,6 +76,7 @@
 def load_tests(standard_tests, module, loader):
     test_bzrdir_implementations = [
         'bzrlib.tests.bzrdir_implementations.test_bzrdir',
+        'bzrlib.tests.bzrdir_implementations.test_push',
         ]
     submod_tests = loader.loadTestsFromModuleNames(test_bzrdir_implementations)
     formats = BzrDirFormat.known_formats()

=== added file 'bzrlib/tests/bzrdir_implementations/test_push.py'
--- bzrlib/tests/bzrdir_implementations/test_push.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/bzrdir_implementations/test_push.py	2009-04-03 14:24:40 +0000
@@ -0,0 +1,65 @@
+# Copyright (C) 2009 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for bzrdir implementations - push."""
+
+from bzrlib import (
+    errors,
+    )
+from bzrlib.tests.bzrdir_implementations import (
+    TestCaseWithBzrDir,
+    )
+
+
+class TestPush(TestCaseWithBzrDir):
+
+    def create_simple_tree(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a'])
+        tree.add(['a'], ['a-id'])
+        tree.commit('one', rev_id='r1')
+        return tree
+
+    def test_push_new_branch(self):     
+        tree = self.create_simple_tree()
+        dir = self.make_repository('dir').bzrdir
+        result = dir.push_branch(tree.branch)
+        self.assertEquals(tree.branch, result.source_branch)
+        self.assertEquals(dir.open_branch().base, result.target_branch.base)
+        self.assertEquals(dir.open_branch().base, 
+            tree.branch.get_push_location())
+
+    def test_push_new_empty(self):
+        tree = self.make_branch_and_tree('tree')
+        dir = self.make_repository('dir').bzrdir
+        result = dir.push_branch(tree.branch)
+        self.assertEquals(tree.branch.base, result.source_branch.base)
+        self.assertEquals(dir.open_branch().base, 
+            result.target_branch.base)
+
+    def test_push_incremental(self):
+        tree = self.create_simple_tree()
+        dir = self.make_repository('dir').bzrdir
+        dir.push_branch(tree.branch)
+        self.build_tree(['tree/b'])
+        tree.add(['b'])
+        tree.commit('two', rev_id='r2')
+        result = dir.push_branch(tree.branch)
+        self.assertEquals(tree.last_revision(), 
+            result.branch_push_result.new_revid)
+        self.assertEquals(2, result.branch_push_result.new_revno)
+        self.assertEquals(tree.branch.base, result.source_branch.base)
+        self.assertEquals(dir.open_branch().base, result.target_branch.base)

=== modified file 'bzrlib/tests/per_repository/test_add_inventory_by_delta.py'
--- bzrlib/tests/per_repository/test_add_inventory_by_delta.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/per_repository/test_add_inventory_by_delta.py	2009-04-03 07:18:11 +0000
@@ -88,3 +88,4 @@
         else:
             repo_delta.commit_write_group()
         self.assertEqual(add_validator, delta_validator)
+        self.assertEqual(new_inv, inv)

=== modified file 'bzrlib/tests/per_repository/test_check.py'
--- bzrlib/tests/per_repository/test_check.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/per_repository/test_check.py	2009-04-03 07:18:11 +0000
@@ -43,7 +43,7 @@
         # XXX: check requires a non-empty revision IDs list, but it ignores the
         # contents of it!
         check_object = tree.branch.repository.check(['ignored'])
-        check_object.report_results(verbose=False)
+        check_object.report_results(verbose=True)
         log = self._get_log(keep_log_file=True)
         self.assertContainsRe(
             log,

=== modified file 'bzrlib/tests/per_repository/test_commit_builder.py'
--- bzrlib/tests/per_repository/test_commit_builder.py	2009-03-30 04:59:50 +0000
+++ bzrlib/tests/per_repository/test_commit_builder.py	2009-04-03 07:18:11 +0000
@@ -185,7 +185,6 @@
             except:
                 builder.abort()
                 raise
-            builder.finish_inventory()
             self.assertEqual(revision_id, builder.commit('foo bar'))
         finally:
             tree.unlock()
@@ -830,12 +829,13 @@
             self.assertEqual(expected_delta, delta)
             builder.finish_inventory()
             rev2 = builder.commit('')
-            tree.set_parent_ids([rev2])
         except:
             builder.abort()
             tree.unlock()
             raise
-        else:
+        try:
+            tree.set_parent_ids([rev2])
+        finally:
             tree.unlock()
         return rev2
 

=== modified file 'bzrlib/tests/per_repository_reference/test_add_inventory.py'
--- bzrlib/tests/per_repository_reference/test_add_inventory.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/per_repository_reference/test_add_inventory.py	2009-04-03 07:18:11 +0000
@@ -30,6 +30,8 @@
         tree = self.make_branch_and_tree('sample')
         revid = tree.commit('one')
         inv = tree.branch.repository.get_inventory(revid)
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
         base = self.make_repository('base')
         repo = self.make_referring('referring', 'base')
         repo.lock_write()
@@ -44,6 +46,10 @@
                 repo.commit_write_group()
         finally:
             repo.unlock()
+        repo.lock_read()
+        self.addCleanup(repo.unlock)
         inv2 = repo.get_inventory(revid)
-        self.assertEqual(inv._byid, inv2._byid)
+        content1 = dict((file_id, inv[file_id]) for file_id in inv)
+        content2 = dict((file_id, inv[file_id]) for file_id in inv2)
+        self.assertEqual(content1, content2)
         self.assertRaises(errors.NoSuchRevision, base.get_inventory, revid)

=== modified file 'bzrlib/tests/per_repository_reference/test_add_revision.py'
--- bzrlib/tests/per_repository_reference/test_add_revision.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/per_repository_reference/test_add_revision.py	2009-04-03 07:18:11 +0000
@@ -30,6 +30,8 @@
         tree = self.make_branch_and_tree('sample')
         revid = tree.commit('one')
         inv = tree.branch.repository.get_inventory(revid)
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
         rev = tree.branch.repository.get_revision(revid)
         base = self.make_repository('base')
         repo = self.make_referring('referring', 'base')

=== modified file 'bzrlib/tests/per_repository_reference/test_add_signature_text.py'
--- bzrlib/tests/per_repository_reference/test_add_signature_text.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/per_repository_reference/test_add_signature_text.py	2009-04-03 07:18:11 +0000
@@ -30,6 +30,8 @@
         tree = self.make_branch_and_tree('sample')
         revid = tree.commit('one')
         inv = tree.branch.repository.get_inventory(revid)
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
         base = self.make_repository('base')
         repo = self.make_referring('referring', 'base')
         repo.lock_write()

=== added file 'bzrlib/tests/test_inventory_delta.py'
--- bzrlib/tests/test_inventory_delta.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_inventory_delta.py	2009-04-02 05:53:12 +0000
@@ -0,0 +1,533 @@
+# Copyright (C) 2008, 2009 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for bzrlib.inventory_delta.
+
+See doc/developer/inventory.txt for more information.
+"""
+
+from cStringIO import StringIO
+
+from bzrlib import (
+    errors,
+    inventory,
+    inventory_delta,
+    )
+from bzrlib.inventory import Inventory
+from bzrlib.revision import NULL_REVISION
+from bzrlib.tests import TestCase
+
+### DO NOT REFLOW THESE TEXTS. NEW LINES ARE SIGNIFICANT. ###
+empty_lines = """format: bzr inventory delta v1 (bzr 1.14)
+parent: null:
+version: null:
+versioned_root: true
+tree_references: true
+"""
+
+root_only_lines = """format: bzr inventory delta v1 (bzr 1.14)
+parent: null:
+version: entry-version
+versioned_root: true
+tree_references: true
+None\x00/\x00an-id\x00\x00a@e\xc3\xa5ample.com--2004\x00dir
+"""
+
+
+root_change_lines = """format: bzr inventory delta v1 (bzr 1.14)
+parent: entry-version
+version: changed-root
+versioned_root: true
+tree_references: true
+/\x00an-id\x00\x00different-version\x00dir
+"""
+
+corrupt_parent_lines = """format: bzr inventory delta v1 (bzr 1.14)
+parent: entry-version
+version: changed-root
+versioned_root: false
+tree_references: false
+/\x00an-id\x00\x00different-version\x00dir
+"""
+
+root_only_unversioned = """format: bzr inventory delta v1 (bzr 1.14)
+parent: null:
+version: entry-version
+versioned_root: false
+tree_references: false
+None\x00/\x00TREE_ROOT\x00\x00null:\x00dir
+"""
+
+reference_lines = """format: bzr inventory delta v1 (bzr 1.14)
+parent: null:
+version: entry-version
+versioned_root: true
+tree_references: true
+None\x00/\x00TREE_ROOT\x00\x00a@e\xc3\xa5ample.com--2004\x00dir
+None\x00/foo\x00id\x00TREE_ROOT\x00changed\x00tree\x00subtree-version
+"""
+
+change_tree_lines = """format: bzr inventory delta v1 (bzr 1.14)
+parent: entry-version
+version: change-tree
+versioned_root: false
+tree_references: false
+/foo\x00id\x00TREE_ROOT\x00changed-twice\x00tree\x00subtree-version2
+"""
+
+
+class TestDeserialization(TestCase):
+    """Test InventoryDeltaSerializer.parse_text_bytes."""
+
+    def test_parse_no_bytes(self):
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=True, tree_references=True)
+        err = self.assertRaises(
+            errors.BzrError, serializer.parse_text_bytes, '')
+        self.assertContainsRe(str(err), 'unknown format')
+
+    def test_parse_bad_format(self):
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=True, tree_references=True)
+        err = self.assertRaises(errors.BzrError,
+            serializer.parse_text_bytes, 'format: foo\n')
+        self.assertContainsRe(str(err), 'unknown format')
+
+    def test_parse_no_parent(self):
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=True, tree_references=True)
+        err = self.assertRaises(errors.BzrError,
+            serializer.parse_text_bytes,
+            'format: bzr inventory delta v1 (bzr 1.14)\n')
+        self.assertContainsRe(str(err), 'missing parent: marker')
+
+    def test_parse_no_version(self):
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=True, tree_references=True)
+        err = self.assertRaises(errors.BzrError,
+            serializer.parse_text_bytes,
+            'format: bzr inventory delta v1 (bzr 1.14)\n'
+            'parent: null:\n')
+        self.assertContainsRe(str(err), 'missing version: marker')
+            
+    def test_parse_duplicate_key_errors(self):
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=True, tree_references=True)
+        double_root_lines = \
+"""format: bzr inventory delta v1 (bzr 1.14)
+parent: null:
+version: null:
+versioned_root: true
+tree_references: true
+None\x00/\x00an-id\x00\x00a@e\xc3\xa5ample.com--2004\x00dir\x00\x00
+None\x00/\x00an-id\x00\x00a@e\xc3\xa5ample.com--2004\x00dir\x00\x00
+"""
+        err = self.assertRaises(errors.BzrError,
+            serializer.parse_text_bytes, double_root_lines)
+        self.assertContainsRe(str(err), 'duplicate file id')
+
+    def test_parse_versioned_root_only(self):
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=True, tree_references=True)
+        parse_result = serializer.parse_text_bytes(root_only_lines)
+        expected_entry = inventory.make_entry(
+            'directory', u'', None, 'an-id')
+        expected_entry.revision = 'a@e\xc3\xa5ample.com--2004'
+        self.assertEqual(
+            ('null:', 'entry-version', [(None, '/', 'an-id', expected_entry)]),
+            parse_result)
+
+    def test_parse_special_revid_not_valid_last_mod(self):
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=False, tree_references=True)
+        root_only_lines = """format: bzr inventory delta v1 (bzr 1.14)
+parent: null:
+version: null:
+versioned_root: false
+tree_references: true
+None\x00/\x00TREE_ROOT\x00\x00null:\x00dir\x00\x00
+"""
+        err = self.assertRaises(errors.BzrError,
+            serializer.parse_text_bytes, root_only_lines)
+        self.assertContainsRe(str(err), 'special revisionid found')
+
+    def test_parse_versioned_root_versioned_disabled(self):
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=False, tree_references=True)
+        root_only_lines = """format: bzr inventory delta v1 (bzr 1.14)
+parent: null:
+version: null:
+versioned_root: false
+tree_references: true
+None\x00/\x00TREE_ROOT\x00\x00a@e\xc3\xa5ample.com--2004\x00dir\x00\x00
+"""
+        err = self.assertRaises(errors.BzrError,
+            serializer.parse_text_bytes, root_only_lines)
+        self.assertContainsRe(str(err), 'Versioned root found')
+
+    def test_parse_unique_root_id_root_versioned_disabled(self):
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=False, tree_references=True)
+        root_only_lines = """format: bzr inventory delta v1 (bzr 1.14)
+parent: null:
+version: null:
+versioned_root: false
+tree_references: true
+None\x00/\x00an-id\x00\x00null:\x00dir\x00\x00
+"""
+        err = self.assertRaises(errors.BzrError,
+            serializer.parse_text_bytes, root_only_lines)
+        self.assertContainsRe(str(err), 'Versioned root found')
+
+    def test_parse_unversioned_root_versioning_enabled(self):
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=True, tree_references=True)
+        err = self.assertRaises(errors.BzrError,
+            serializer.parse_text_bytes, root_only_unversioned)
+        self.assertContainsRe(
+            str(err), 'serialized versioned_root flag is wrong: False')
+
+    def test_parse_tree_when_disabled(self):
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=True, tree_references=False)
+        err = self.assertRaises(errors.BzrError,
+            serializer.parse_text_bytes, reference_lines)
+        self.assertContainsRe(
+            str(err), 'serialized tree_references flag is wrong: True')
+
+
+class TestSerialization(TestCase):
+    """Tests for InventoryDeltaSerializer.delta_to_lines."""
+
+    def test_empty_delta_to_lines(self):
+        old_inv = Inventory(None)
+        new_inv = Inventory(None)
+        delta = new_inv._make_delta(old_inv)
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=True, tree_references=True)
+        self.assertEqual(StringIO(empty_lines).readlines(),
+            serializer.delta_to_lines(NULL_REVISION, NULL_REVISION, delta))
+
+    def test_root_only_to_lines(self):
+        old_inv = Inventory(None)
+        new_inv = Inventory(None)
+        root = new_inv.make_entry('directory', '', None, 'an-id')
+        root.revision = 'a@e\xc3\xa5ample.com--2004'
+        new_inv.add(root)
+        delta = new_inv._make_delta(old_inv)
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=True, tree_references=True)
+        self.assertEqual(StringIO(root_only_lines).readlines(),
+            serializer.delta_to_lines(NULL_REVISION, 'entry-version', delta))
+
+    def test_unversioned_root(self):
+        old_inv = Inventory(None)
+        new_inv = Inventory(None)
+        root = new_inv.make_entry('directory', '', None, 'TREE_ROOT')
+        new_inv.add(root)
+        delta = new_inv._make_delta(old_inv)
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=False, tree_references=False)
+        self.assertEqual(StringIO(root_only_unversioned).readlines(),
+            serializer.delta_to_lines(NULL_REVISION, 'entry-version', delta))
+
+    def test_unversioned_non_root_errors(self):
+        old_inv = Inventory(None)
+        new_inv = Inventory(None)
+        root = new_inv.make_entry('directory', '', None, 'TREE_ROOT')
+        root.revision = 'a@e\xc3\xa5ample.com--2004'
+        new_inv.add(root)
+        non_root = new_inv.make_entry('directory', 'foo', root.file_id, 'id')
+        new_inv.add(non_root)
+        delta = new_inv._make_delta(old_inv)
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=True, tree_references=True)
+        err = self.assertRaises(errors.BzrError,
+            serializer.delta_to_lines, NULL_REVISION, 'entry-version', delta)
+        self.assertEqual(str(err), 'no version for fileid id')
+
+    def test_richroot_unversioned_root_errors(self):
+        old_inv = Inventory(None)
+        new_inv = Inventory(None)
+        root = new_inv.make_entry('directory', '', None, 'TREE_ROOT')
+        new_inv.add(root)
+        delta = new_inv._make_delta(old_inv)
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=True, tree_references=True)
+        err = self.assertRaises(errors.BzrError,
+            serializer.delta_to_lines, NULL_REVISION, 'entry-version', delta)
+        self.assertEqual(str(err), 'no version for fileid TREE_ROOT')
+
+    def test_nonrichroot_versioned_root_errors(self):
+        old_inv = Inventory(None)
+        new_inv = Inventory(None)
+        root = new_inv.make_entry('directory', '', None, 'TREE_ROOT')
+        root.revision = 'a@e\xc3\xa5ample.com--2004'
+        new_inv.add(root)
+        delta = new_inv._make_delta(old_inv)
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=False, tree_references=True)
+        err = self.assertRaises(errors.BzrError,
+            serializer.delta_to_lines, NULL_REVISION, 'entry-version', delta)
+        self.assertEqual(str(err), 'Version present for / in TREE_ROOT')
+
+    def test_nonrichroot_non_TREE_ROOT_id_errors(self):
+        old_inv = Inventory(None)
+        new_inv = Inventory(None)
+        root = new_inv.make_entry('directory', '', None, 'my-rich-root-id')
+        new_inv.add(root)
+        delta = new_inv._make_delta(old_inv)
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=False, tree_references=True)
+        err = self.assertRaises(errors.BzrError,
+            serializer.delta_to_lines, NULL_REVISION, 'entry-version', delta)
+        self.assertEqual(
+            str(err), 'file_id my-rich-root-id is not TREE_ROOT for /')
+
+    def test_unknown_kind_errors(self):
+        old_inv = Inventory(None)
+        new_inv = Inventory(None)
+        root = new_inv.make_entry('directory', '', None, 'my-rich-root-id')
+        root.revision = 'changed'
+        new_inv.add(root)
+        non_root = new_inv.make_entry('directory', 'foo', root.file_id, 'id')
+        non_root.revision = 'changed'
+        non_root.kind = 'strangelove'
+        new_inv.add(non_root)
+        delta = new_inv._make_delta(old_inv)
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=True, tree_references=True)
+        # we expect keyerror because there is little value wrapping this.
+        # This test aims to prove that it errors more than how it errors.
+        err = self.assertRaises(KeyError,
+            serializer.delta_to_lines, NULL_REVISION, 'entry-version', delta)
+        self.assertEqual(('strangelove',), err.args)
+
+    def test_tree_reference_disabled(self):
+        old_inv = Inventory(None)
+        new_inv = Inventory(None)
+        root = new_inv.make_entry('directory', '', None, 'TREE_ROOT')
+        root.revision = 'a@e\xc3\xa5ample.com--2004'
+        new_inv.add(root)
+        non_root = new_inv.make_entry(
+            'tree-reference', 'foo', root.file_id, 'id')
+        non_root.revision = 'changed'
+        non_root.reference_revision = 'subtree-version'
+        new_inv.add(non_root)
+        delta = new_inv._make_delta(old_inv)
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=True, tree_references=False)
+        # we expect keyerror because there is little value wrapping this.
+        # This test aims to prove that it errors more than how it errors.
+        err = self.assertRaises(KeyError,
+            serializer.delta_to_lines, NULL_REVISION, 'entry-version', delta)
+        self.assertEqual(('tree-reference',), err.args)
+
+    def test_tree_reference_enabled(self):
+        old_inv = Inventory(None)
+        new_inv = Inventory(None)
+        root = new_inv.make_entry('directory', '', None, 'TREE_ROOT')
+        root.revision = 'a@e\xc3\xa5ample.com--2004'
+        new_inv.add(root)
+        non_root = new_inv.make_entry(
+            'tree-reference', 'foo', root.file_id, 'id')
+        non_root.revision = 'changed'
+        non_root.reference_revision = 'subtree-version'
+        new_inv.add(non_root)
+        delta = new_inv._make_delta(old_inv)
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=True, tree_references=True)
+        self.assertEqual(StringIO(reference_lines).readlines(),
+            serializer.delta_to_lines(NULL_REVISION, 'entry-version', delta))
+
+    def test_to_inventory_root_id_versioned_not_permitted(self):
+        delta = [(None, '/', 'TREE_ROOT', inventory.make_entry(
+            'directory', '', None, 'TREE_ROOT'))]
+        serializer = inventory_delta.InventoryDeltaSerializer(False, True)
+        self.assertRaises(
+            errors.BzrError, serializer.delta_to_lines, 'old-version',
+            'new-version', delta)
+
+    def test_to_inventory_root_id_not_versioned(self):
+        delta = [(None, '/', 'an-id', inventory.make_entry(
+            'directory', '', None, 'an-id'))]
+        serializer = inventory_delta.InventoryDeltaSerializer(True, True)
+        self.assertRaises(
+            errors.BzrError, serializer.delta_to_lines, 'old-version',
+            'new-version', delta)
+
+    def test_to_inventory_has_tree_not_meant_to(self):
+        make_entry = inventory.make_entry
+        tree_ref = make_entry('tree-reference', 'foo', 'changed-in', 'ref-id')
+        tree_ref.reference_revision = 'ref-revision'
+        delta = [
+            (None, '/', 'an-id',
+             make_entry('directory', '', 'changed-in', 'an-id')),
+            (None, '/foo', 'ref-id', tree_ref)
+            # a file that followed the root move
+            ]
+        serializer = inventory_delta.InventoryDeltaSerializer(True, True)
+        self.assertRaises(errors.BzrError, serializer.delta_to_lines,
+            'old-version', 'new-version', delta)
+
+    def test_to_inventory_torture(self):
+        def make_entry(kind, name, parent_id, file_id, **attrs):
+            entry = inventory.make_entry(kind, name, parent_id, file_id)
+            for name, value in attrs.items():
+                setattr(entry, name, value)
+            return entry
+        # this delta is crafted to have all the following:
+        # - deletes
+        # - renamed roots
+        # - deep dirs
+        # - files moved after parent dir was renamed
+        # - files with and without exec bit
+        delta = [
+            # new root:
+            (None, '', 'new-root-id',
+                make_entry('directory', '', None, 'new-root-id',
+                    revision='changed-in')),
+            # an old root:
+            ('', 'old-root', 'TREE_ROOT',
+                make_entry('directory', 'subdir-now', 'new-root-id',
+                'TREE_ROOT', revision='moved-root')),
+            # a file that followed the root move
+            ('under-old-root', 'old-root/under-old-root', 'moved-id',
+                make_entry('file', 'under-old-root', 'TREE_ROOT', 'moved-id',
+                   revision='old-rev', executable=False, text_size=30,
+                   text_sha1='some-sha')),
+            # a deleted path
+            ('old-file', None, 'deleted-id', None),
+            # a tree reference moved to the new root
+            ('ref', 'ref', 'ref-id',
+                make_entry('tree-reference', 'ref', 'new-root-id', 'ref-id',
+                    reference_revision='tree-reference-id',
+                    revision='new-rev')),
+            # a symlink now in a deep dir
+            ('dir/link', 'old-root/dir/link', 'link-id',
+                make_entry('symlink', 'link', 'deep-id', 'link-id',
+                   symlink_target='target', revision='new-rev')),
+            # a deep dir
+            ('dir', 'old-root/dir', 'deep-id',
+                make_entry('directory', 'dir', 'TREE_ROOT', 'deep-id',
+                    revision='new-rev')),
+            # a file with an exec bit set
+            (None, 'configure', 'exec-id',
+                make_entry('file', 'configure', 'new-root-id', 'exec-id',
+                   executable=True, text_size=30, text_sha1='some-sha',
+                   revision='old-rev')),
+            ]
+        serializer = inventory_delta.InventoryDeltaSerializer(True, True)
+        lines = serializer.delta_to_lines(NULL_REVISION, 'something', delta)
+        expected = """format: bzr inventory delta v1 (bzr 1.14)
+parent: null:
+version: something
+versioned_root: true
+tree_references: true
+/\x00/old-root\x00TREE_ROOT\x00new-root-id\x00moved-root\x00dir
+/dir\x00/old-root/dir\x00deep-id\x00TREE_ROOT\x00new-rev\x00dir
+/dir/link\x00/old-root/dir/link\x00link-id\x00deep-id\x00new-rev\x00link\x00target
+/old-file\x00None\x00deleted-id\x00\x00null:\x00deleted\x00\x00
+/ref\x00/ref\x00ref-id\x00new-root-id\x00new-rev\x00tree\x00tree-reference-id
+/under-old-root\x00/old-root/under-old-root\x00moved-id\x00TREE_ROOT\x00old-rev\x00file\x0030\x00\x00some-sha
+None\x00/\x00new-root-id\x00\x00changed-in\x00dir
+None\x00/configure\x00exec-id\x00new-root-id\x00old-rev\x00file\x0030\x00Y\x00some-sha
+"""
+        serialized = ''.join(lines)
+        self.assertIsInstance(serialized, str)
+        self.assertEqual(expected, serialized)
+
+
+class TestContent(TestCase):
+    """Test serialization of the content part of a line."""
+
+    def test_dir(self):
+        entry = inventory.make_entry('directory', 'a dir', None)
+        self.assertEqual('dir', inventory_delta._directory_content(entry))
+
+    def test_file_0_short_sha(self):
+        file_entry = inventory.make_entry('file', 'a file', None, 'file-id')
+        file_entry.text_sha1 = ''
+        file_entry.text_size = 0
+        self.assertEqual('file\x000\x00\x00',
+            inventory_delta._file_content(file_entry))
+
+    def test_file_10_foo(self):
+        file_entry = inventory.make_entry('file', 'a file', None, 'file-id')
+        file_entry.text_sha1 = 'foo'
+        file_entry.text_size = 10
+        self.assertEqual('file\x0010\x00\x00foo',
+            inventory_delta._file_content(file_entry))
+
+    def test_file_executable(self):
+        file_entry = inventory.make_entry('file', 'a file', None, 'file-id')
+        file_entry.executable = True
+        file_entry.text_sha1 = 'foo'
+        file_entry.text_size = 10
+        self.assertEqual('file\x0010\x00Y\x00foo',
+            inventory_delta._file_content(file_entry))
+
+    def test_file_without_size(self):
+        file_entry = inventory.make_entry('file', 'a file', None, 'file-id')
+        file_entry.text_sha1 = 'foo'
+        self.assertRaises(errors.BzrError,
+            inventory_delta._file_content, file_entry)
+
+    def test_file_without_sha1(self):
+        file_entry = inventory.make_entry('file', 'a file', None, 'file-id')
+        file_entry.text_size = 10
+        self.assertRaises(errors.BzrError,
+            inventory_delta._file_content, file_entry)
+
+    def test_link_empty_target(self):
+        entry = inventory.make_entry('symlink', 'a link', None)
+        entry.symlink_target = ''
+        self.assertEqual('link\x00',
+            inventory_delta._link_content(entry))
+
+    def test_link_unicode_target(self):
+        entry = inventory.make_entry('symlink', 'a link', None)
+        entry.symlink_target = ' \xc3\xa5'.decode('utf8')
+        self.assertEqual('link\x00 \xc3\xa5',
+            inventory_delta._link_content(entry))
+
+    def test_link_space_target(self):
+        entry = inventory.make_entry('symlink', 'a link', None)
+        entry.symlink_target = ' '
+        self.assertEqual('link\x00 ',
+            inventory_delta._link_content(entry))
+
+    def test_link_no_target(self):
+        entry = inventory.make_entry('symlink', 'a link', None)
+        self.assertRaises(errors.BzrError,
+            inventory_delta._link_content, entry)
+
+    def test_reference_null(self):
+        entry = inventory.make_entry('tree-reference', 'a tree', None)
+        entry.reference_revision = NULL_REVISION
+        self.assertEqual('tree\x00null:',
+            inventory_delta._reference_content(entry))
+
+    def test_reference_revision(self):
+        entry = inventory.make_entry('tree-reference', 'a tree', None)
+        entry.reference_revision = 'foo@\xc3\xa5b-lah'
+        self.assertEqual('tree\x00foo@\xc3\xa5b-lah',
+            inventory_delta._reference_content(entry))
+
+    def test_reference_no_reference(self):
+        entry = inventory.make_entry('tree-reference', 'a tree', None)
+        self.assertRaises(errors.BzrError,
+            inventory_delta._reference_content, entry)

=== modified file 'bzrlib/tests/test_remote.py'
--- bzrlib/tests/test_remote.py	2009-04-02 03:37:30 +0000
+++ bzrlib/tests/test_remote.py	2009-04-03 01:21:06 +0000
@@ -1332,16 +1332,9 @@
         self.assertEqual('rejection message', err.msg)
 
 
-class TestBranchControlGetBranchConf(RemoteBranchTestCase):
-    """Getting the branch configuration should use an abstract method not vfs.
-    """
+class TestBranchGetSetConfig(RemoteBranchTestCase):
 
     def test_get_branch_conf(self):
-        # We should see that branch.get_config() does a single rpc to get the
-        # remote configuration file, abstracting away where that is stored on
-        # the server.  However at the moment it always falls back to using the
-        # vfs, and this would need some changes in config.py.
-
         # in an empty branch we decode the response properly
         client = FakeClient()
         client.add_expected_call(
@@ -1357,6 +1350,55 @@
              ('call_expecting_body', 'Branch.get_config_file', ('memory:///',))],
             client._calls)
 
+    def test_get_multi_line_branch_conf(self):
+        # Make sure that multiple-line branch.conf files are supported
+        #
+        # https://bugs.edge.launchpad.net/bzr/+bug/354075
+        client = FakeClient()
+        client.add_expected_call(
+            'Branch.get_stacked_on_url', ('memory:///',),
+            'error', ('NotStacked',),)
+        client.add_success_response_with_body('a = 1\nb = 2\nc = 3\n', 'ok')
+        transport = MemoryTransport()
+        branch = self.make_remote_branch(transport, client)
+        config = branch.get_config()
+        self.assertEqual(u'2', config.get_user_option('b'))
+
+    def test_set_option(self):
+        client = FakeClient()
+        client.add_expected_call(
+            'Branch.get_stacked_on_url', ('memory:///',),
+            'error', ('NotStacked',),)
+        client.add_expected_call(
+            'Branch.lock_write', ('memory:///', '', ''),
+            'success', ('ok', 'branch token', 'repo token'))
+        client.add_expected_call(
+            'Branch.set_config_option', ('memory:///', 'branch token',
+            'repo token', 'foo', 'bar', ''),
+            'success', ())
+        client.add_expected_call(
+            'Branch.unlock', ('memory:///', 'branch token', 'repo token'),
+            'success', ('ok',))
+        transport = MemoryTransport()
+        branch = self.make_remote_branch(transport, client)
+        branch.lock_write()
+        config = branch._get_config()
+        config.set_option('foo', 'bar')
+        branch.unlock()
+        client.finished_test()
+
+    def test_backwards_compat_set_option(self):
+        self.setup_smart_server_with_call_log()
+        branch = self.make_branch('.')
+        verb = 'Branch.set_config_option'
+        self.disable_verb(verb)
+        branch.lock_write()
+        self.addCleanup(branch.unlock)
+        self.reset_smart_call_log()
+        branch._get_config().set_option('value', 'name')
+        self.assertLength(10, self.hpss_calls)
+        self.assertEqual('value', branch._get_config().get_option('name'))
+
 
 class TestBranchLockWrite(RemoteBranchTestCase):
 

=== modified file 'bzrlib/tests/test_rename_map.py'
--- bzrlib/tests/test_rename_map.py	2009-03-24 12:54:05 +0000
+++ bzrlib/tests/test_rename_map.py	2009-03-30 20:59:42 +0000
@@ -17,6 +17,7 @@
 
 import os
 
+from bzrlib import trace
 from bzrlib.rename_map import RenameMap
 from bzrlib.tests import TestCaseWithTransport
 
@@ -158,3 +159,43 @@
         os.rename('tree/foo', 'tree/baz')
         RenameMap.guess_renames(tree)
         self.assertEqual('baz/empty', tree.id2path('empty-id'))
+
+    def test_guess_renames_dry_run(self):
+        tree = self.make_branch_and_tree('tree')
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        self.build_tree(['tree/file'])
+        tree.add('file', 'file-id')
+        tree.commit('Added file')
+        os.rename('tree/file', 'tree/file2')
+        RenameMap.guess_renames(tree, dry_run=True)
+        self.assertEqual('file', tree.id2path('file-id'))
+
+    @staticmethod
+    def captureNotes(cmd, *args, **kwargs):
+        notes = []
+        def my_note(fmt, *args):
+            notes.append(fmt % args)
+        old_note = trace.note
+        trace.note = my_note
+        try:
+            result = cmd(*args, **kwargs)
+        finally:
+            trace.note = old_note
+        return notes, result
+
+    def test_guess_renames_output(self):
+        """guess_renames emits output whether dry_run is True or False."""
+        tree = self.make_branch_and_tree('tree')
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        self.build_tree(['tree/file'])
+        tree.add('file', 'file-id')
+        tree.commit('Added file')
+        os.rename('tree/file', 'tree/file2')
+        notes = self.captureNotes(RenameMap.guess_renames, tree,
+                                  dry_run=True)[0]
+        self.assertEqual('file => file2', ''.join(notes))
+        notes = self.captureNotes(RenameMap.guess_renames, tree,
+                                  dry_run=False)[0]
+        self.assertEqual('file => file2', ''.join(notes))

=== modified file 'bzrlib/tests/test_smart.py'
--- bzrlib/tests/test_smart.py	2009-03-24 23:19:12 +0000
+++ bzrlib/tests/test_smart.py	2009-04-02 05:22:42 +0000
@@ -502,7 +502,41 @@
             request.execute(''))
 
 
-class SetLastRevisionTestBase(tests.TestCaseWithMemoryTransport):
+class TestLockedBranch(tests.TestCaseWithMemoryTransport):
+
+    def get_lock_tokens(self, branch):
+        branch_token = branch.lock_write()
+        repo_token = branch.repository.lock_write()
+        branch.repository.unlock()
+        return branch_token, repo_token
+
+
+class TestSmartServerBranchRequestSetConfigOption(TestLockedBranch):
+
+    def test_value_name(self):
+        branch = self.make_branch('.')
+        request = smart.branch.SmartServerBranchRequestSetConfigOption(
+            branch.bzrdir.root_transport)
+        branch_token, repo_token = self.get_lock_tokens(branch)
+        config = branch._get_config()
+        result = request.execute('', branch_token, repo_token, 'bar', 'foo',
+            '')
+        self.assertEqual(SuccessfulSmartServerResponse(()), result)
+        self.assertEqual('bar', config.get_option('foo'))
+
+    def test_value_name_section(self):
+        branch = self.make_branch('.')
+        request = smart.branch.SmartServerBranchRequestSetConfigOption(
+            branch.bzrdir.root_transport)
+        branch_token, repo_token = self.get_lock_tokens(branch)
+        config = branch._get_config()
+        result = request.execute('', branch_token, repo_token, 'bar', 'foo',
+            'gam')
+        self.assertEqual(SuccessfulSmartServerResponse(()), result)
+        self.assertEqual('bar', config.get_option('foo', 'gam'))
+
+
+class SetLastRevisionTestBase(TestLockedBranch):
     """Base test case for verbs that implement set_last_revision."""
 
     def setUp(self):
@@ -512,11 +546,7 @@
         self.tree = self.make_branch_and_memory_tree('.')
 
     def lock_branch(self):
-        b = self.tree.branch
-        branch_token = b.lock_write()
-        repo_token = b.repository.lock_write()
-        b.repository.unlock()
-        return branch_token, repo_token
+        return self.get_lock_tokens(self.tree.branch)
 
     def unlock_branch(self):
         self.tree.branch.unlock()
@@ -1377,6 +1407,9 @@
             smart.request.request_handlers.get('Branch.revision_history'),
             smart.branch.SmartServerRequestRevisionHistory)
         self.assertEqual(
+            smart.request.request_handlers.get('Branch.set_config_option'),
+            smart.branch.SmartServerBranchRequestSetConfigOption)
+        self.assertEqual(
             smart.request.request_handlers.get('Branch.set_last_revision'),
             smart.branch.SmartServerBranchRequestSetLastRevision)
         self.assertEqual(

=== modified file 'bzrlib/tests/test_version_info.py'
--- bzrlib/tests/test_version_info.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_version_info.py	2009-04-03 21:23:55 +0000
@@ -53,6 +53,16 @@
 
         return wt
 
+    def test_rio_null(self):
+        wt = self.make_branch_and_tree('branch')
+
+        sio = StringIO()
+        builder = RioVersionInfoBuilder(wt.branch, working_tree=wt)
+        builder.generate(sio)
+        val = sio.getvalue()
+        self.assertContainsRe(val, 'build-date:')
+        self.assertContainsRe(val, 'revno: 0')
+
     def test_rio_version_text(self):
         wt = self.create_branch()
 
@@ -154,6 +164,16 @@
         self.assertEqual(['r4', 'r4', 'unversioned', 'removed'],
                          file_rev_stanza.get_all('revision'))
 
+    def test_python_null(self):
+        wt = self.make_branch_and_tree('branch')
+
+        sio = StringIO()
+        builder = PythonVersionInfoBuilder(wt.branch, working_tree=wt)
+        builder.generate(sio)
+        val = sio.getvalue()
+        self.assertContainsRe(val, "'revision_id': None")
+        self.assertContainsRe(val, "'revno': 0")
+
     def test_python_version(self):
         wt = self.create_branch()
 
@@ -229,6 +249,20 @@
         self.assertEqual('unversioned', tvi.file_revisions['c'])
         self.assertEqual('removed', tvi.file_revisions['d'])
 
+    def test_custom_null(self):
+        sio = StringIO()
+        wt = self.make_branch_and_tree('branch')
+        builder = CustomVersionInfoBuilder(wt.branch, working_tree=wt,
+            template='revno: {revno}')
+        builder.generate(sio)
+        self.assertEquals("revno: 0", sio.getvalue())
+
+        builder = CustomVersionInfoBuilder(wt.branch, working_tree=wt, 
+            template='{revno} revid: {revision_id}')
+        # revision_id is not available yet
+        self.assertRaises(errors.MissingTemplateVariable, 
+            builder.generate, sio)
+
     def test_custom_version_text(self):
         wt = self.create_branch()
 

=== modified file 'bzrlib/tests/test_versionedfile.py'
--- bzrlib/tests/test_versionedfile.py	2009-03-24 01:53:42 +0000
+++ bzrlib/tests/test_versionedfile.py	2009-04-03 00:53:29 +0000
@@ -24,9 +24,9 @@
 from itertools import chain, izip
 from StringIO import StringIO
 
-import bzrlib
 from bzrlib import (
     errors,
+    knit as _mod_knit,
     osutils,
     progress,
     )
@@ -35,7 +35,6 @@
                            RevisionAlreadyPresent,
                            WeaveParentMismatch
                            )
-from bzrlib import knit as _mod_knit
 from bzrlib.knit import (
     cleanup_pack_knit,
     make_file_factory,
@@ -179,7 +178,7 @@
 
 
 def get_diamond_files(files, key_length, trailing_eol=True, left_only=False,
-    nograph=False):
+    nograph=False, nokeys=False):
     """Get a diamond graph to exercise deltas and merges.
 
     This creates a 5-node graph in files. If files supports 2-length keys two
@@ -192,8 +191,12 @@
     :param nograph: If True, do not provide parents to the add_lines calls;
         this is useful for tests that need inserted data but have graphless
         stores.
+    :param nokeys: If True, pass None is as the key for all insertions.
+        Currently implies nograph.
     :return: The results of the add_lines calls.
     """
+    if nokeys:
+        nograph = True
     if key_length == 1:
         prefixes = [()]
     else:
@@ -210,25 +213,30 @@
         else:
             result = [prefix + suffix for suffix in suffix_list]
             return result
+    def get_key(suffix):
+        if nokeys:
+            return (None, )
+        else:
+            return (suffix,)
     # we loop over each key because that spreads the inserts across prefixes,
     # which is how commit operates.
     for prefix in prefixes:
-        result.append(files.add_lines(prefix + ('origin',), (),
+        result.append(files.add_lines(prefix + get_key('origin'), (),
             ['origin' + last_char]))
     for prefix in prefixes:
-        result.append(files.add_lines(prefix + ('base',),
+        result.append(files.add_lines(prefix + get_key('base'),
             get_parents([('origin',)]), ['base' + last_char]))
     for prefix in prefixes:
-        result.append(files.add_lines(prefix + ('left',),
+        result.append(files.add_lines(prefix + get_key('left'),
             get_parents([('base',)]),
             ['base\n', 'left' + last_char]))
     if not left_only:
         for prefix in prefixes:
-            result.append(files.add_lines(prefix + ('right',),
+            result.append(files.add_lines(prefix + get_key('right'),
                 get_parents([('base',)]),
                 ['base\n', 'right' + last_char]))
         for prefix in prefixes:
-            result.append(files.add_lines(prefix + ('merged',),
+            result.append(files.add_lines(prefix + get_key('merged'),
                 get_parents([('left',), ('right',)]),
                 ['base\n', 'left\n', 'right\n', 'merged' + last_char]))
     return result
@@ -1487,10 +1495,11 @@
         """Each parameterised test can be constructed on a transport."""
         files = self.get_versionedfiles()
 
-    def get_diamond_files(self, files, trailing_eol=True, left_only=False):
+    def get_diamond_files(self, files, trailing_eol=True, left_only=False,
+        nokeys=False):
         return get_diamond_files(files, self.key_length,
             trailing_eol=trailing_eol, nograph=not self.graph,
-            left_only=left_only)
+            left_only=left_only, nokeys=nokeys)
 
     def test_add_lines_nostoresha(self):
         """When nostore_sha is supplied using old content raises."""
@@ -1544,6 +1553,60 @@
                 ('ed8bce375198ea62444dc71952b22cfc2b09226d', 23)],
                 results)
 
+    def test_add_lines_no_key_generates_chk_key(self):
+        files = self.get_versionedfiles()
+        # save code by using the stock data insertion helper.
+        adds = self.get_diamond_files(files, nokeys=True)
+        results = []
+        # We can only validate the first 2 elements returned from add_lines.
+        for add in adds:
+            self.assertEqual(3, len(add))
+            results.append(add[:2])
+        if self.key_length == 1:
+            self.assertEqual([
+                ('00e364d235126be43292ab09cb4686cf703ddc17', 7),
+                ('51c64a6f4fc375daf0d24aafbabe4d91b6f4bb44', 5),
+                ('a8478686da38e370e32e42e8a0c220e33ee9132f', 10),
+                ('9ef09dfa9d86780bdec9219a22560c6ece8e0ef1', 11),
+                ('ed8bce375198ea62444dc71952b22cfc2b09226d', 23)],
+                results)
+            # Check the added items got CHK keys.
+            self.assertEqual(set([
+                ('sha1:00e364d235126be43292ab09cb4686cf703ddc17',),
+                ('sha1:51c64a6f4fc375daf0d24aafbabe4d91b6f4bb44',),
+                ('sha1:9ef09dfa9d86780bdec9219a22560c6ece8e0ef1',),
+                ('sha1:a8478686da38e370e32e42e8a0c220e33ee9132f',),
+                ('sha1:ed8bce375198ea62444dc71952b22cfc2b09226d',),
+                ]),
+                files.keys())
+        elif self.key_length == 2:
+            self.assertEqual([
+                ('00e364d235126be43292ab09cb4686cf703ddc17', 7),
+                ('00e364d235126be43292ab09cb4686cf703ddc17', 7),
+                ('51c64a6f4fc375daf0d24aafbabe4d91b6f4bb44', 5),
+                ('51c64a6f4fc375daf0d24aafbabe4d91b6f4bb44', 5),
+                ('a8478686da38e370e32e42e8a0c220e33ee9132f', 10),
+                ('a8478686da38e370e32e42e8a0c220e33ee9132f', 10),
+                ('9ef09dfa9d86780bdec9219a22560c6ece8e0ef1', 11),
+                ('9ef09dfa9d86780bdec9219a22560c6ece8e0ef1', 11),
+                ('ed8bce375198ea62444dc71952b22cfc2b09226d', 23),
+                ('ed8bce375198ea62444dc71952b22cfc2b09226d', 23)],
+                results)
+            # Check the added items got CHK keys.
+            self.assertEqual(set([
+                ('FileA', 'sha1:00e364d235126be43292ab09cb4686cf703ddc17'),
+                ('FileA', 'sha1:51c64a6f4fc375daf0d24aafbabe4d91b6f4bb44'),
+                ('FileA', 'sha1:9ef09dfa9d86780bdec9219a22560c6ece8e0ef1'),
+                ('FileA', 'sha1:a8478686da38e370e32e42e8a0c220e33ee9132f'),
+                ('FileA', 'sha1:ed8bce375198ea62444dc71952b22cfc2b09226d'),
+                ('FileB', 'sha1:00e364d235126be43292ab09cb4686cf703ddc17'),
+                ('FileB', 'sha1:51c64a6f4fc375daf0d24aafbabe4d91b6f4bb44'),
+                ('FileB', 'sha1:9ef09dfa9d86780bdec9219a22560c6ece8e0ef1'),
+                ('FileB', 'sha1:a8478686da38e370e32e42e8a0c220e33ee9132f'),
+                ('FileB', 'sha1:ed8bce375198ea62444dc71952b22cfc2b09226d'),
+                ]),
+                files.keys())
+
     def test_empty_lines(self):
         """Empty files can be stored."""
         f = self.get_versionedfiles()
@@ -1591,8 +1654,9 @@
         for factory in entries:
             on_seen(factory.key)
             self.assertValidStorageKind(factory.storage_kind)
-            self.assertEqual(f.get_sha1s([factory.key])[factory.key],
-                factory.sha1)
+            if factory.sha1 is not None:
+                self.assertEqual(f.get_sha1s([factory.key])[factory.key],
+                    factory.sha1)
             self.assertEqual(parents[factory.key], factory.parents)
             self.assertIsInstance(factory.get_bytes_as(factory.storage_kind),
                 str)
@@ -1735,8 +1799,9 @@
         for factory in entries:
             seen.add(factory.key)
             self.assertValidStorageKind(factory.storage_kind)
-            self.assertEqual(files.get_sha1s([factory.key])[factory.key],
-                factory.sha1)
+            if factory.sha1 is not None:
+                self.assertEqual(files.get_sha1s([factory.key])[factory.key],
+                                 factory.sha1)
             self.assertEqual(parent_map[factory.key], factory.parents)
             # currently no stream emits mpdiff
             self.assertRaises(errors.UnavailableRepresentation,
@@ -1940,8 +2005,9 @@
                 self.assertEqual(None, factory.parents)
             else:
                 self.assertValidStorageKind(factory.storage_kind)
-                self.assertEqual(files.get_sha1s([factory.key])[factory.key],
-                    factory.sha1)
+                if factory.sha1 is not None:
+                    sha1 = files.get_sha1s([factory.key])[factory.key]
+                    self.assertEqual(sha1, factory.sha1)
                 self.assertEqual(parents[factory.key], factory.parents)
                 self.assertIsInstance(factory.get_bytes_as(factory.storage_kind),
                     str)

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2009-03-24 01:53:42 +0000
+++ bzrlib/tree.py	2009-04-03 01:24:32 +0000
@@ -905,7 +905,6 @@
             output. An unversioned file is defined as one with (False, False)
             for the versioned pair.
         """
-        result = []
         lookup_trees = [self.source]
         if extra_trees:
              lookup_trees.extend(extra_trees)
@@ -977,6 +976,10 @@
                 if (self.source.get_symlink_target(file_id) !=
                     self.target.get_symlink_target(file_id)):
                     changed_content = True
+                # XXX: Yes, the indentation below is wrong. But fixing it broke
+                # test_merge.TestMergerEntriesLCAOnDisk.
+                # test_nested_tree_subtree_renamed_and_modified. We'll wait for
+                # the fix from bzr.dev -- vila 2009026
                 elif from_kind == 'tree-reference':
                     if (self.source.get_reference_revision(file_id, from_path)
                         != self.target.get_reference_revision(file_id, to_path)):

=== modified file 'bzrlib/ui/__init__.py'
--- bzrlib/ui/__init__.py	2009-03-23 14:59:43 +0000
+++ bzrlib/ui/__init__.py	2009-04-02 17:41:41 +0000
@@ -157,7 +157,6 @@
         self.stderr = stderr or sys.stderr
 
     def get_boolean(self, prompt):
-        self.clear_term()
         # FIXME: make a regexp and handle case variations as well.
         while True:
             self.prompt(prompt + "? [y/n]: ")
@@ -167,11 +166,20 @@
             if line in ('n\n', 'no\n'):
                 return False
 
-    def get_non_echoed_password(self, prompt):
-        if not sys.stdin.isatty():
-            raise errors.NotATerminal()
-        encoding = osutils.get_terminal_encoding()
-        return getpass.getpass(prompt.encode(encoding, 'replace'))
+    def get_non_echoed_password(self):
+        isatty = getattr(self.stdin, 'isatty', None)
+        if isatty is not None and isatty():
+            # getpass() ensure the password is not echoed and other
+            # cross-platform niceties
+            password = getpass.getpass('')
+        else:
+            # echo doesn't make sense without a terminal
+            password = self.stdin.readline()
+            if not password:
+                password = None
+            elif password[-1] == '\n':
+                password = password[:-1]
+        return password
 
     def get_password(self, prompt='', **kwargs):
         """Prompt the user for a password.
@@ -184,13 +192,16 @@
                  canceled the request.
         """
         prompt += ': '
-        prompt = (prompt % kwargs)
+        self.prompt(prompt, **kwargs)
         # There's currently no way to say 'i decline to enter a password'
         # as opposed to 'my password is empty' -- does it matter?
-        return self.get_non_echoed_password(prompt)
+        return self.get_non_echoed_password()
 
-    def prompt(self, prompt):
+    def prompt(self, prompt, **kwargs):
         """Emit prompt on the CLI."""
+        prompt = prompt % kwargs
+        prompt = prompt.encode(osutils.get_terminal_encoding(), 'replace')
+        self.clear_term()
         self.stdout.write(prompt)
 
     def note(self, msg):
@@ -210,7 +221,7 @@
     def get_password(self, prompt='', **kwargs):
         return None
 
-    def prompt(self, prompt):
+    def prompt(self, prompt, **kwargs):
         pass
 
     def note(self, msg):

=== modified file 'bzrlib/ui/text.py'
--- bzrlib/ui/text.py	2009-03-23 14:59:43 +0000
+++ bzrlib/ui/text.py	2009-04-02 10:43:01 +0000
@@ -60,10 +60,6 @@
         # paints progress, network activity, etc
         self._progress_view = TextProgressView(self.stderr)
 
-    def prompt(self, prompt):
-        """Emit prompt on the CLI."""
-        self.stdout.write(prompt)
-
     def clear_term(self):
         """Prepare the terminal for output.
 

=== modified file 'bzrlib/version_info_formats/format_custom.py'
--- bzrlib/version_info_formats/format_custom.py	2009-03-23 14:59:43 +0000
+++ bzrlib/version_info_formats/format_custom.py	2009-04-03 21:23:55 +0000
@@ -18,6 +18,9 @@
    tree info."""
 
 from bzrlib import errors
+from bzrlib.revision import (
+   NULL_REVISION,
+   )
 from bzrlib.lazy_regex import lazy_compile
 from bzrlib.version_info_formats import (
    create_date_str,
@@ -87,7 +90,7 @@
         info.add('branch_nick', self._branch.nick)
 
         revision_id = self._get_revision_id()
-        if revision_id is None:
+        if revision_id == NULL_REVISION:
             info.add('revno', 0)
         else:
             info.add('revno', self._branch.revision_id_to_revno(revision_id))

=== modified file 'bzrlib/version_info_formats/format_python.py'
--- bzrlib/version_info_formats/format_python.py	2009-03-23 14:59:43 +0000
+++ bzrlib/version_info_formats/format_python.py	2009-04-03 21:23:55 +0000
@@ -18,6 +18,9 @@
 
 import pprint
 
+from bzrlib.revision import (
+    NULL_REVISION,
+    )
 from bzrlib.version_info_formats import (
     create_date_str,
     VersionInfoBuilder,
@@ -57,7 +60,7 @@
         revisions = []
 
         revision_id = self._get_revision_id()
-        if revision_id is None:
+        if revision_id == NULL_REVISION:
             info['revno'] = 0
         else:
             info['revno'] = self._branch.revision_id_to_revno(revision_id)

=== modified file 'bzrlib/version_info_formats/format_rio.py'
--- bzrlib/version_info_formats/format_rio.py	2009-03-23 14:59:43 +0000
+++ bzrlib/version_info_formats/format_rio.py	2009-04-03 21:23:55 +0000
@@ -16,6 +16,9 @@
 
 """A generator which creates a rio stanza of the current tree info"""
 
+from bzrlib.revision import (
+    NULL_REVISION,
+    )
 from bzrlib.rio import RioWriter, Stanza
 
 from bzrlib.version_info_formats import (
@@ -30,7 +33,7 @@
     def generate(self, to_file):
         info = Stanza()
         revision_id = self._get_revision_id()
-        if revision_id is not None:
+        if revision_id != NULL_REVISION:
             info.add('revision-id', revision_id)
             rev = self._branch.repository.get_revision(revision_id)
             info.add('date', create_date_str(rev.timestamp, rev.timezone))

=== modified file 'bzrlib/versionedfile.py'
--- bzrlib/versionedfile.py	2009-03-23 14:59:43 +0000
+++ bzrlib/versionedfile.py	2009-04-03 00:53:29 +0000
@@ -794,7 +794,8 @@
         check_content=True):
         """Add a text to the store.
 
-        :param key: The key tuple of the text to add.
+        :param key: The key tuple of the text to add. If the last element is
+            None, a CHK string will be generated during the addition.
         :param parents: The parents key tuples of the text to add.
         :param lines: A list of lines. Each line must be a bytestring. And all
             of them except the last must be terminated with \n and contain no

=== modified file 'bzrlib/weave.py'
--- bzrlib/weave.py	2009-03-24 01:53:42 +0000
+++ bzrlib/weave.py	2009-04-03 00:53:29 +0000
@@ -410,6 +410,7 @@
         version_id
             Symbolic name for this version.
             (Typically the revision-id of the revision that added it.)
+            If None, a name will be allocated based on the hash. (sha1:SHAHASH)
 
         parents
             List or set of direct parent version numbers.
@@ -425,6 +426,8 @@
             sha1 = sha_strings(lines)
         if sha1 == nostore_sha:
             raise errors.ExistingContent
+        if version_id is None:
+            version_id = "sha1:" + sha1
         if version_id in self._name_map:
             return self._check_repeated_add(version_id, parents, lines, sha1)
 

=== modified file 'bzrlib/workingtree_4.py'
--- bzrlib/workingtree_4.py	2009-03-31 14:13:27 +0000
+++ bzrlib/workingtree_4.py	2009-04-03 01:24:32 +0000
@@ -1466,6 +1466,10 @@
                            _control_files=control_files)
 
     def __get_matchingbzrdir(self):
+        return self._get_matchingbzrdir()
+
+    def _get_matchingbzrdir(self):
+        """Overrideable method to get a bzrdir for testing."""
         # please test against something that will let us do tree references
         return bzrdir.format_registry.make_bzrdir(
             'dirstate-with-subtree')

=== modified file 'bzrlib/xml_serializer.py'
--- bzrlib/xml_serializer.py	2009-04-02 15:28:11 +0000
+++ bzrlib/xml_serializer.py	2009-04-03 23:18:11 +0000
@@ -166,3 +166,18 @@
         elementtree.ElementTree._raise_serialization_error(text)
 
 elementtree.ElementTree._escape_cdata = _escape_cdata
+
+
+def escape_invalid_chars(message):
+    """Escape the XML-invalid characters in a commit message.
+
+    :param message: Commit message to escape
+    :param count: Number of characters that were escaped
+    """
+    # Python strings can include characters that can't be
+    # represented in well-formed XML; escape characters that
+    # aren't listed in the XML specification
+    # (http://www.w3.org/TR/REC-xml/#NT-Char).
+    return re.subn(u'[^\x09\x0A\x0D\u0020-\uD7FF\uE000-\uFFFD]+',
+            lambda match: match.group(0).encode('unicode_escape'),
+            message)

=== modified file 'doc/developers/development-repo.txt'
--- doc/developers/development-repo.txt	2008-10-28 22:19:31 +0000
+++ doc/developers/development-repo.txt	2009-04-03 01:34:58 +0000
@@ -257,6 +257,12 @@
 These formats use B+Tree indices which are considerably faster than
 the earlier indices in bzr.
 
+Development5[Subtree]
+---------------------
+
+These formats use B+Tree indices and also offer a CHK based byte store
+in addition to the revision/inventory/signatures and text stores.
+
 
 ..
    vim: tw=72 ft=rst expandtab

=== added file 'doc/developers/groupcompress-design.txt'
--- doc/developers/groupcompress-design.txt	1970-01-01 00:00:00 +0000
+++ doc/developers/groupcompress-design.txt	2009-04-03 01:34:58 +0000
@@ -0,0 +1,115 @@
+This document contains notes about the design for groupcompress, replacement
+VersionedFiles store for use in pack based repositories. The goal is to provide
+fast, history bounded text extraction.
+
+Overview
+++++++++
+
+The goal: Much tighter compression, maintained automatically. Considerations
+to weigh: The minimum IO to reconstruct a text with no other repository
+involved; The number of index lookups to plan a reconstruction. The minimum
+IO to reconstruct a text with another repositories assistance (affects
+network IO for fetch, which impacts incremental pulls and shallow branch
+operations).
+
+Current approach
+================
+
+Each delta is individually compressed against another text, and then entropy
+compressed. We index the pointers between these deltas.
+
+Solo reconstruction: Plan a readv via the index, read the deltas in forward
+IO, apply each delta. Total IO: sum(deltas) + deltacount*index overhead.
+Fetch/stacked reconstruction: Plan a readv via the index, using local basis
+texts where possible. Then readv locally and remote and apply deltas. Total IO
+as for solo reconstruction.
+
+Things to keep
+==============
+
+Reasonable sizes 'amount read' from remote machines to reconstruct an arbitrary
+text: Reading 5MB for a 100K plain text is not a good trade off. Reading (say)
+500K is probably acceptable. Reading ~100K is ideal. However, its likely that
+some texts (e.g NEWS versions) can be stored for nearly-no space at all if we
+are willing to have unbounded IO. Profiling to set a good heuristic will be
+important. Also allowing users to choose to optimise for a server environment
+may make sense: paying more local IO for less compact storage may be useful.
+
+Things to remove
+================
+
+Index scatter gather IO. Doing hundreds or thousands of index lookups is very
+expensive, and doing that per file just adds insult to injury.
+
+Partioned compression amongst files.
+
+Scatter gather IO when reconstructing texts: linear forward IO is better.
+
+Thoughts
+========
+
+Merges combine texts from multiple versions to create a new version. Deltas
+add new text to existing files and remove some text from the same. Getting
+high compression means reading some base and then a chain of deltas (could
+be a tree) to gain access to the thing that the final delta was made against,
+and that delta. Rather than composing all these deltas, we can just just
+perform the final diff against the base text and the serialised invidual
+deltas. If the diff algorithm can reuse out of order lines from previous
+texts (e.g. storing AB -> BA as pointers rather than delete and add, then
+the presence of any previously stored line in a single chain can be reused.
+One such diff algorithm is xdelta, another reasonable one to consider is
+plain old zlib or lzma. We could also use bzip2. One advantage of using
+a generic compression engine is less python code. One advantage of
+preprocessing line based deltas is that we reduce the window size for the
+text repeated within lines, and that will help compression by a simple
+entropy compressor as a post processor.
+lzma appears fantastic at compression - 420MB of NEWS files down to 200KB.
+so window size appears to be a key determiner for efficiency.
+
+Delta strategy
+++++++++++++++
+
+Very big objects - no delta. I plan to kick this in at 5MB initially, but
+once the codebase is up and running, we can tweak this to
+
+Very small objects - no delta? If they are combined with a larger zlib object
+why not? (Answer: because zlib's window is really small)
+
+Other objects - group by fileid (gives related texts a chance, though using a
+file name would be better long term as e.g. COPYING and COPYING from different
+projects could combine). Then by reverse topological graph(as this places more
+recent texts at the front of a chain). Alternatively, group by size, though
+that should not matter with a large enough window.
+Finally, delta the texts against the current output of the compressor. This is
+essentially a somewhat typed form of sliding window dictionary compression. An
+alternative implementation would be to just use zlib, or lzma, or bzip2
+directory.
+
+Unfortunately, just using entropy compression forces a lot of data to be output
+by the decompressor - e.g. 420MB in the NEWS sample corpus. When we only want
+a single 55K text thats inefficient. (An initial test took several seconds with
+lzma.)
+
+The fastest to implement approach is probably just 'diff output to date and add
+to entropy compressor'. This should produce reasonable results. As delta
+chain length is not a concern (only one delta to apply ever), we can simply
+cap the chain when the total read size becomes unreasonable. Given older texts
+are smaller we probably want some weighted factor of plaintext size.
+
+In this approach, a single entropy compressed region is read as a unit, giving
+the lower bound for IO (and how much to read is an open question - what byte
+offset of compressed data is sufficient to ensue that the delta-stream contents
+we need are reconstructable. Flushing, while possible, degrades compression(and
+adds overhead - we'd be paying 4 bytes per record guaranteed). Again - tests
+will be needed.
+
+A nice possibility is to output mpdiff compatible records, which might enable
+some code reuse. This is more work than just diff (current_out, new_text), so
+can wait for the concept to be proven.
+
+Implementation Strategy
++++++++++++++++++++++++
+
+Bring up a VersionedFiles object that implements this, then stuff it into a
+repository format. zlib as a starting compressor, though bzip2 will probably
+do a good job.

=== modified file 'doc/developers/index.txt'
--- doc/developers/index.txt	2009-03-25 17:05:22 +0000
+++ doc/developers/index.txt	2009-04-03 01:34:58 +0000
@@ -77,8 +77,8 @@
 * `Container format <container-format.html>`_ |--| Notes on a container format
   for streaming and storing Bazaar data.
 
-* `Repository stream <repository-stream.html>`_ |--| Notes on streaming data
-  for repositories (a layer above the container format).
+* `Groupcompress <groupcompress-design.html>`_ |--| Notes on the compression
+  technology used in CHK repositories.
 
 * `Indices <indices.html>`_ |--| The index facilities available within bzrlib.
 
@@ -92,6 +92,9 @@
 
 * `Repositories <repository.html>`_ |--| What repositories do and are used for.
 
+* `Repository stream <repository-stream.html>`_ |--| Notes on streaming data
+  for repositories (a layer above the container format).
+
 * `Integration Guide <integration.html>`_ |--| A guide to integrate bzrlib into
   any python application.
 

=== modified file 'doc/developers/inventory.txt'
--- doc/developers/inventory.txt	2008-09-24 07:33:56 +0000
+++ doc/developers/inventory.txt	2009-04-02 05:53:12 +0000
@@ -451,3 +451,50 @@
 
 Multiple versions of nodes for the same PREFIX and internal prefix width should
 compress well for the same tree.
+
+
+Inventory deltas
+================
+
+An inventory is a serialization of the in-memory inventory delta.  To serialize
+an inventory delta, one takes an existing inventory delta and the revision_id
+of the revision it was created it against and the revision id of the inventory
+which should result by applying the delta to the parent.  We then serialize
+every item in the delta in a simple format:
+
+'format: bzr inventory delta v1 (1.14)' NL
+'parent:' SP BASIS_INVENTORY NL
+'version:' SP NULL_OR_REVISION NL
+'versioned_root:' SP BOOL NL
+'tree_references:' SP BOOL NL
+DELTA_LINES
+
+DELTA_LINES ::= (DELTA_LINE NL)*
+DELTA_LINE ::= OLDPATH NULL NEWPATH NULL file-id NULL PARENT_ID NULL LAST_MODIFIED NULL CONTENT
+SP ::= ' '
+BOOL ::= 'true' | 'false'
+NULL ::= \x00
+OLDPATH ::= NONE | PATH
+NEWPATH ::= NONE | PATH
+NONE ::= 'None'
+PATH ::= path
+PARENT_ID ::= FILE_ID | ''
+CONTENT ::= DELETED_CONTENT | FILE_CONTENT | DIR_CONTENT | TREE_CONTENT | LINK_CONTENT
+DELETED_CONTENT ::= 'deleted'
+FILE_CONTENT ::= 'file' NULL text_size NULL EXEC NULL text_sha1
+DIR_CONTENT ::= 'dir'
+TREE_CONTENT ::= 'tree' NULL tree-revision
+LINK_CONTENT ::= 'link' NULL link-target
+BASIS_INVENTORY ::= NULL_OR_REVISION
+LAST_MODIFIED ::= NULL_OR_REVISION
+NULL_OR_REVISION ::= 'null:' | REVISION
+REVISION ::= revision-id-in-utf8-no-whitespace
+EXEC ::= '' | 'Y'
+
+DELTA_LINES is lexicographically sorted.
+
+Some explanation is in order. When NEWPATH is 'None' a delete has been
+recorded, and because this inventory delta is not attempting to be a reversible
+delta, the only other valid fields are OLDPATH and 'file-id'. PARENT_ID is ''
+when a delete has been recorded or when recording a new root entry.
+

=== modified file 'doc/en/user-guide/svn_plugin.txt'
--- doc/en/user-guide/svn_plugin.txt	2008-12-16 04:07:30 +0000
+++ doc/en/user-guide/svn_plugin.txt	2009-03-30 21:28:51 +0000
@@ -105,17 +105,12 @@
 
 Bazaar and Subversion are different tools with different capabilities
 so there will always be some limited interoperability issues.
-Here are some examples current as of bzr-svn 0.4.10:
+Here are some examples current as of bzr-svn 0.5.4:
 
  * Bazaar doesn't support versioned properties
 
  * Bazaar doesn't support tracking of file copies.
 
- * ``bzr push`` to an existing Subversion branch works but 
-   ``bzr push`` to a new Subversion branch doesn't. Until
-   the relevant bug is fixed (#121875), you need to use
-   ``bzr svn-push`` in the latter case.
-
 See the bzr-svn web page,
 http://bazaar-vcs.org/BzrForeignBranches/Subversion,
 for the current list of constraints.

=== modified file 'tools/win32/build_release.py'
--- tools/win32/build_release.py	2009-02-24 20:43:40 +0000
+++ tools/win32/build_release.py	2009-03-28 16:10:36 +0000
@@ -8,6 +8,7 @@
     'qbzr': '0.9.8',
     'bzrtools': '1.12.0',
     'bzr-svn': '0.5.2',
+    'bzr-rebase': '0.4.4',
     'subvertpy': '0.6.4',
 }
 
@@ -124,7 +125,7 @@
 
 
 def _plugin_tag_name(plugin_name):
-    if plugin_name in ('bzr-svn', 'subvertpy'):
+    if plugin_name in ('bzr-svn', 'bzr-rebase', 'subvertpy'):
         return '%s-%s' % (plugin_name, VERSIONS[plugin_name])
     # bzrtools and qbzr use 'release-X.Y.Z'
     return 'release-' + VERSIONS[plugin_name]
@@ -133,7 +134,7 @@
 def update_plugin(plugin_name):
     release_dir = get_plugin_release_dir(plugin_name)
     if not os.path.isdir(plugin_name):
-        if plugin_name == 'bzr-svn':
+        if plugin_name in ('bzr-svn', 'bzr-rebase'):
             # bzr-svn uses a different repo format
             call_or_fail([bzr(), 'init-repo', '--rich-root-pack', plugin_name])
         else:
@@ -194,6 +195,7 @@
     install_plugin('bzrtools')
     install_plugin('qbzr')
     install_plugin('bzr-svn')
+    install_plugin('bzr-rebase')
 
     build_installer()
 

